# $Id: ServerErrorInfo.rb,v 1.1 2005/02/05 14:35:35 toki Exp $

require 'forwardable'

class ServerErrorInfo < WPM::PageContext
  extend Forwardable

  def init_context
    @control = page_option(:control)
    params = driver.params
    @error_type = params['type']
    @error_pos = params['id'].to_i
    case (@error_type)
    when 'load'
      @error_sample = @control.setup_load_errors[@error_pos]
    when 'logging'
      @error_sample = @control.setup_logging_errors[@error_pos]
    when 'alias'
      @error_sample = @control.setup_alias_errors[@error_pos]
    when 'document'
      @error_sample = @control.setup_document_errors[@error_pos]
    when 'filter'
      @error_sample = @control.setup_filter_errors[@error_pos]
    else
      raise "unknown error type: #{@error_type.inspect}"
    end
    @frame_info = nil
    @arg_value = nil
  end

  def reload_query
    { 'type' => @error_type,
      'id' => @error_pos.to_s
    }
  end

  def load_error?
    @error_type == 'load'
  end

  def logging_error?
    @error_type == 'logging'
  end

  def alias_error?
    @error_type == 'alias'
  end

  def document_error?
    @error_type == 'document'
  end

  def filter_error?
    @error_type == 'filter'
  end

  # for Rucy::RuntimeErrorSample class
  def_delegator :@error_sample, :error_message
  def_delegator :@error_sample, :error_type
  def_delegator :@error_sample, :backtrace
  def_delegator :@error_sample, :timestamp
  attr_accessor :frame_info

  def has_backtrace?
    @error_sample.backtrace ? true : false
  end

  # for Rucy::LoadErrorSample class
  def_delegator :@error_sample, :mod_path

  # for Rucy::LoggingErrorSample class
  def_delegator :@error_sample, :log_path

  # for Rucy::AliasErrorSample class
  def_delegator :@error_sample, :alias_path
  def_delegator :@error_sample, :orig_path
  def_delegator :@error_sample, :virtual_host

  # for Rucy::DocumentErrorSample class
  def_delegator :@error_sample, :document
  def_delegator :@error_sample, :arguments
  def_delegator :@error_sample, :mount_path
  def_delegator :@error_sample, :mount_mask
  #def_delegator :@error_sample, :virtual_host # defined at Rucy::AliasErrorSample class

  # for Rucy::FilterErrorSample class
  def_delegator :@error_sample, :filter
  #def_delegator :@error_sample, :arguments # defined at Rucy::DocumentErrorSample class
  def_delegator :@error_sample, :attach_path
  def_delegator :@error_sample, :attach_mask
  #def_delegator :@error_sample, :virtual_host # defined at Rucy::AliasErrorSample class

  attr_accessor :arg_value
end
