# $Id: rc_d.rb,v 1.7 2005/02/11 16:25:13 toki Exp $

require 'rucy/rep'

module Rucy
  module Daemon
    def daemon
      catch(:START_DAEMON) do
	unless (fork) then
	  Process.setsid
	  unless (fork) then
	    #Dir.chdir('/')
	    STDIN.reopen('/dev/null', 'r')
	    STDOUT.reopen('/dev/null', 'w')
	    STDERR.reopen('/dev/null', 'w')
	    throw :START_DAEMON
	  end
	end
	exit!
      end

      nil
    end
    module_function :daemon
  end

  module ProcessProbe
    def proc_alive?(pid)
      begin
	Process.kill(0, pid)
	return true
      rescue Errno::EPERM
	return true
      rescue SystemCallError
	return false
      end

      raise 'internal error'
    end
    module_function :proc_alive?
  end

  class RC_d
    LOCK_FILE = File.join(File.dirname($0), 'rucy.pid')

    def initialize(lock, rep=Repository::DEFAULT)
      @lock = lock
      @rep = rep
      @File = @rep.fetch(:File_class)
      @Signal = @rep.fetch(:Signal_module)
      @Daemon = @rep.fetch(:Daemon_module)
      @Process = @rep.fetch(:Process_module)
      @ProcessProbe = @rep.fetch(:ProcessProbe_module)
    end

    def read_pid
      @File.open(@lock, 'r') {|input| input.read }.to_i
    end
    private :read_pid

    # for server

    def server_setup(server)
      @Signal.trap('TERM') { server.close }
      if (server.daemon) then
	@Signal.trap('HUP') { server.restart }
	@Daemon.daemon
      end

      nil
    end

    def write_pid
      @File.open(@lock, File::WRONLY | File::CREAT | File::TRUNC) {|output|
	output.write($$.to_s + "\n")
      }
      nil
    end

    def lock
      begin
	@File.open(@lock, File::WRONLY | File::CREAT | File::TRUNC | File::EXCL) {|output|
	  output.write($$.to_s + "\n")
	}
      rescue Errno::EEXIST
	pid = read_pid
	if (@ProcessProbe.proc_alive? pid) then
	  raise 'running server'
	else
	  # race condition (^_^;)
	  # don't parallel lock!
	  @File.delete(@lock)
	  retry
	end
      end

      nil
    end

    # for /etc/rc.d

    def running?
      begin
	pid = read_pid
	if (@ProcessProbe.proc_alive? pid) then
	  return true
	else
	  return false
	end
      rescue Errno::ENOENT
	return false
      end

      raise 'internal error'
    end

    def stop
      pid = read_pid
      @Process.kill('TERM', pid)
      nil
    end
  end
end
