# $Id: cgidriver.rb,v 1.4 2005/06/16 14:04:06 toki Exp $
# -
# HTTP message driver for cgi.rb

=begin
# example

require 'wpm'
require 'wpm/rexml'
require 'wpm/cgidriver'

writer = WPM::Writer.new(WPM::XMLAssistByREXML)
writer.run(WPM::CGIDriver.new)
=end

require 'cgi'

module WPM
  class CGIDriver < Driver
    def initialize(cgi=CGI.new)
      @cgi = cgi
      @options = Hash.new
      @messg_body = ''
      @closed = false
    end

    def handler
      @closed and raise 'closed'
      @cgi
    end

    def env
      @closed and raise 'closed'
      ENV
    end

    def params
      @closed and raise 'closed'

      params = Hash.new
      for name in @cgi.keys
	params[name] = @cgi.params[name].first.to_s
      end

      params
    end

    def header(name)
      @closed and raise 'closed'

      case (name.upcase)
      when 'CONTENT-TYPE'
	content_type = @options['type']
	content_type += '; charset=' + @options['charset'] if (@options.include? 'charset')
	return content_type
      when 'STATUS'
	return @options['status']
      when 'SERVER'
	return @options['server']
      when 'CONNECTION'
	return @options['connection']
      when 'CONTENT-LENGTH'
	return @options['length']
      when 'CONTENT-LANGUAGE'
	return @options['language']
      when 'EXPIRES'
	return @options['expires']
      when 'SET-COOKIE'
	return @options['cookie']
      else
	return @options[name]
      end
    end

    def set_header(name, value)
      @closed and raise 'closed'

      case (name.upcase)
      when 'CONTENT-TYPE'
	if (value =~ %r"^\S+/\S+") then
	  @options['type'] = $&
	  @options['type'].gsub!(/;$/, '')
	end
	if (value =~ %r'charset="([^"]+)"' || value =~ %r"charset=(\S+)") then
	  @options['charset'] = $1
	end
      when 'STATUS'
	@options['status'] = value
      when 'SERVER'
	@options['server'] = value
      when 'CONNECTION'
	@options['connection'] = value
      when 'CONTENT-LENGTH'
	@options['length'] = value
      when 'CONTENT-LANGUAGE'
	@options['language'] = value
      when 'EXPIRES'
	@options['expires'] = value
      when 'SET-COOKIE'
	@options['cookie'] = value
      else
	@options[name] = value
      end

      nil
    end

    def write(messg_body)
      @closed and raise 'closed'
      @messg_body.concat(messg_body)
      nil
    end

    def close
      @closed and raise 'closed'
      @closed = true
      @cgi.out(@options) { @messg_body }
      nil
    end
  end
end
