# $Id: rucydriver.rb,v 1.12 2005/06/16 15:29:45 toki Exp $
# -
# HTTP message driver for Rucy

require 'forwardable'
require 'rucy/logger'
require 'rucy/document'

module WPM
  class RucyDriver < Driver
    def initialize(script_name, request, response)
      @request = request
      @response = response
      @write_body = false
      @env = @request.cgi_env(script_name)
      @params = Hash.new
      case (@env['REQUEST_METHOD'])
      when 'GET'
	query_string = @request.query
      when 'POST'
	if (@env['CONTENT_TYPE'].downcase == 'application/x-www-form-urlencoded') then
	  query_string = @request.fetch_body
	else
	  query_string = nil
	end
      else
	query_string = nil
      end
      if (query_string) then
	for param_string in query_string.split(/&/, -1)
	  name, value = param_string.split(/=/, 2)
	  if (value) then
	    name = Escape.unescapeURL(name)
	    unless (@params.include? name) then
	      @params[name] = Escape.unescapeURL(value)
	    end
	  end
	end
      end
    end

    def handler
      [ @request, @response ]
    end

    attr_reader :env
    attr_reader :params

    def header(name)
      case (name.upcase)
      when 'STATUS'
	return "#{@response.status} #{@response.reason}"
      else
	return @response.header(name)
      end
    end

    def set_header(name, value)
      case (name.upcase)
      when 'STATUS'
	code, reason = value.split(/\s+/, 2)
	@response.status = code.to_i
	@response.reason = reason
      else
	@response.set_header(name, value)
      end

      nil
    end

    def check_write_body
      unless (@write_body) then
	@response.start_body
	@write_body = true
      end
      nil
    end
    private :check_write_body

    def write(messg_body)
      check_write_body
      @response.write(messg_body)
      nil
    end

    def close
      check_write_body
      nil
    end
  end
end

module Rucy
  class WebPageMakerDocument < Document
    def initialize(writer)
      @writer = writer
    end

    def publish(script_name, request, response, logger)
      logger.debug("enter document: #{self.class}")
      driver = WPM::RucyDriver.new(script_name, request, response)
      @writer.run(driver)
      hit = @writer.prof[:hit]
      unhit = @writer.prof[:unhit]
      logger.debug("#{script_name}: page cache report: hit/unhit = #{hit}/#{unhit}")
      nil
    end
  end
end
