# $Id: webrickdriver.rb,v 1.4 2005/06/16 15:29:48 toki Exp $
# -
# HTTP message driver for WEBrick

=begin
# example

require 'wpm'
require 'wpm/rexml'
require 'wpm/webrickdriver'
writer = WPM::Writer.new(WPM::XMLAssistByREXML)
writer.base_dir = '/path_of_application'

require 'webrick'
httpd = WEBrick::HTTPServer.new({ :Port => 8888 })
httpd.mount('/test', WEBrick::HTTPServlet::WPMHandler, writer)
trap(:INT){ httpd.shutdown }
httpd.start
=end

require 'forwardable'
require 'webrick/httpstatus'
require 'webrick/httpservlet/abstract'

module WPM
  class WEBrickDriver < Driver
    extend Forwardable

    def initialize(req, res)
      @req = req
      @res = res
    end

    def handler
      [ @req, @res ]
    end

    def_delegator :@req, :meta_vars, :env
    def_delegator :@req, :query, :params

    def header(name)
      case (name.upcase)
      when 'STATUS'
	return "#{@res.status} #{WEBrick::HTTPStatus::reason_phrase(@res.status)}"
      else
	return @res[name]
      end
    end

    def set_header(name, value)
      case (name.upcase)
      when 'STATUS'
	code, reason = value.split(/\s+/, 2)
	@res.status = code.to_i
      else
	@res[name] = value
      end

      nil
    end

    def write(messg_body)
      @res.body << messg_body
      nil
    end

    def close
      nil
    end
  end
end

module WEBrick
  module HTTPServlet
    class WPMHandler < AbstractServlet
      def initialize(server, writer)
	super
	@writer = writer
      end

      def service(req, res)
	driver = WPM::WEBrickDriver.new(req, res)
	@writer.run(driver)
	hit = @writer.prof[:hit]
	unhit = @writer.prof[:unhit]
	@logger.debug("#{req.script_name}: page cache report: hit/unhit = #{hit}/#{unhit}")
	nil
      end
    end
  end
end
