#!/usr/local/bin/ruby
# $Id: rucy.rb,v 1.16 2004/12/25 02:00:36 toki Exp $

require 'getoptlong'
getoptlong = GetoptLong.new([ '--help', '-h', GetoptLong::NO_ARGUMENT ],
			    [ '--version', '-v', GetoptLong::NO_ARGUMENT ],
			    [ '--daemon', '-d', GetoptLong::NO_ARGUMENT ],
			    [ '--store-path', GetoptLong::REQUIRED_ARGUMENT ])

usage = <<EOF
usage: ruby #{File.basename($0)} [options]
options:
  --help, -h           show this message.
  --version, -v        show version.
  --daemon, -d         start as background daemon process.
  --store-path=[db]    change database path to store server parameters.
Start and see `http://localhost:8888/control' (default control panel).
EOF

show_help = false
show_version = false
daemon = false

base_dir = File.expand_path(File.dirname($0))
lib_core = File.join(base_dir, 'lib_core')
lib_ctrl = File.join(base_dir, 'lib_ctrl')
lib_conf = File.join(base_dir, 'lib_conf')
lib_wpm = File.join(base_dir, 'lib_wpm')
mod_docs = File.join(base_dir, 'mod_docs')
store_path = File.join(base_dir, 'control.db')
page_path = File.join(base_dir, 'control')

for opt_name, opt_arg in getoptlong
  case (opt_name)
  when '--help', '-h'
    show_help = true
  when '--version', '-v'
    show_version = true
  when '--daemon', '-d'
    daemon = true
  when '--store-path'
    store_path = opt_arg
  end
end

$:.push(lib_core)
$:.push(lib_ctrl)
$:.push(lib_conf)
$:.push(lib_wpm)

require 'rucy/version'
print Rucy::RUCY_TOKEN, "\n" if show_version
print usage if show_help
exit 1 if (show_help || show_version)

require 'rucy/rep'
require 'rucy/config'
rep = Rucy::Repository::DEFAULT
server = rep.fetch(:Server)
server.daemon = daemon
server.open_hook{|s|
  control = rep.fetch(:Control)
  control.load_path = mod_docs
  control.store_path = store_path
  control.page_path = page_path
  control.server_setup(s)
}
rc_d = rep.fetch(:RC_d)
rc_d.server_setup(server)
rc_d.write_pid
server.accept
