#!/usr/local/bin/ruby
# $Id: test_AccessLog.rb,v 1.8 2005/01/16 11:08:15 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'wpm'

module TestRucy
  class TestAccessLog < RUNIT::TestCase
    def setup
      # for WPM::Page class
      @page = Forwarder.new(self)

      # for Rucy::Control class
      @access_log_params_call = 0
      @set_access_log_params_call = 0
      @set_access_log_params_value = nil

      @control = Forwarder.new(self)
      class << @control
	def_delegator :__getobj__, :access_log_params
	def_delegator :__getobj__, :set_access_log_params
      end

      # for Rucy::AccessLogParams class
      @modified_count_call = 0
      @set_modified_count_call = 0
      @set_modified_count_value = nil
      @logging_call = 0
      @set_logging_call = 0
      @set_logging_flag = nil
      @format_call = 0
      @set_format_call = 0
      @set_format_value = nil
      @logfiles_call = 0
      @del_logfile_at_call = 0
      @del_logfile_at_pos = nil

      @access_log_params = Forwarder.new(self)
      class << @access_log_params
	def_delegator :__getobj__, :modified_count
	def_delegator :__getobj__, :modified_count=
	def_delegator :__getobj__, :logging
	def_delegator :__getobj__, :logging=
	def_delegator :__getobj__, :format
	def_delegator :__getobj__, :format=
	def_delegator :__getobj__, :logfiles
	def_delegator :__getobj__, :del_logfile_at
      end

      # for Rucy::AccessLogParams::LogFile class
      @logfile_path_call = 0
      @set_logfile_path_call = 0
      @set_logfile_path_value = nil
      @logfile_format_call = 0
      @set_logfile_format_call = 0
      @set_logfile_format_value = nil

      @logfile = Forwarder.new(self)
      class << @logfile
	def_delegator :__getobj__, :logfile_path, :path
	def_delegator :__getobj__, :logfile_path=, :path=
	def_delegator :__getobj__, :logfile_format, :format
	def_delegator :__getobj__, :logfile_format=, :format=
      end
      @logfile2 = Object.new	# dummy
      @logfiles_list = [ @logfile, @logfile2 ]

      # setup WPM::PageContext class
      loader = WPM::Loader.new('../control/AccessLog/AccessLog.rb')
      option = { :control => @control }
      @AccessLog = loader.const_get('AccessLog').new(@page, option)
      assert((@AccessLog.is_a? WPM::PageContext))
    end

    # for Rucy::Control class

    def access_log_params
      @access_log_params_call += 1
      @access_log_params
    end

    def set_access_log_params(params)
      @set_access_log_params_call += 1
      @set_access_log_params_value = params
      nil
    end

    # for Rucy::AccessLogParams class

    def modified_count
      @modified_count_call += 1
      7
    end

    def modified_count=(new_count)
      @set_modified_count_call += 1
      @set_modified_count_value = new_count
    end

    def logging
      @logging_call += 1
      true
    end

    def logging=(flag)
      @set_logging_call += 1
      @set_logging_flag = flag
    end

    def format
      @format_call += 1
      'foo'
    end

    def format=(value)
      @set_format_call += 1
      @set_format_value = value
    end

    def logfiles
      @logfiles_call += 1
      @logfiles_list.dup
    end

    def del_logfile_at(pos)
      @del_logfile_at_call += 1
      @del_logfile_at_pos = pos
      nil
    end

    # for Rucy::AccessLogParams::LogFile class

    def logfile_path
      @logfile_path_call += 1
      'access.log'
    end

    def logfile_path=(new_path)
      @set_logfile_path_call += 1
      @set_logfile_path_value = new_path
    end

    def logfile_format
      @logfile_format_call += 1
      'foo'
    end

    def logfile_format=(new_format)
      @set_logfile_format_call += 1
      @set_logfile_format_value = new_format
    end

    # test

    def init_context
      @AccessLog.init_context
      assert_equal(1, @access_log_params_call)
      assert_equal(0, @logfiles_call)
    end

    def test_modified_count
      init_context
      assert_equal(7, @AccessLog.modified_count)
      assert_equal(1, @modified_count_call)
      @AccessLog.modified_count = 8
      assert_equal(1, @set_modified_count_call)
      assert_equal(8, @set_modified_count_value)
    end

    def test_logging
      init_context
      assert_equal(true, @AccessLog.logging)
      assert_equal(1, @logging_call)
    end

    def test_set_logging
      init_context
      @AccessLog.logging = false
      assert_equal(1, @set_logging_call)
      assert_equal(false, @set_logging_flag)
    end

    def test_format
      init_context
      assert_equal('foo', @AccessLog.format)
      assert_equal(1, @format_call)
    end

    def test_set_format
      init_context
      @AccessLog.format = 'bar'
      assert_equal(1, @set_format_call)
      assert_equal('bar', @set_format_value)
    end

    def test_has_logfiles_true
      init_context
      assert_equal(true, @AccessLog.has_logfiles?)
      assert_equal(1, @logfiles_call)
    end

    def test_has_logfiles_false
      @logfiles_list.clear
      init_context
      assert_equal(false, @AccessLog.has_logfiles?)
      assert_equal(1, @logfiles_call)
    end

    def test_index
      @AccessLog.index = 1
      assert_equal(1, @AccessLog.index)
    end

    def test_logfile_list
      init_context
      assert_equal([ { :pos => 0, :params => @logfile },
		     { :pos => 1, :params => @logfile2 }
		   ], @AccessLog.logfile_list)
      assert_equal(1, @logfiles_call)
    end

    def test_logfile_path
      init_context
      @AccessLog.logfile = { :params => @logfile }
      assert_equal('access.log', @AccessLog.logfile_path)
      assert_equal(1, @logfile_path_call)
    end

    def test_set_logfile_path
      init_context
      @AccessLog.logfile = { :params => @logfile }
      @AccessLog.logfile_path = 'access2.log'
      assert_equal(1, @set_logfile_path_call)
      assert_equal('access2.log', @set_logfile_path_value)
    end

    def test_logfile_format
      init_context
      @AccessLog.logfile = { :params => @logfile }
      assert_equal('foo', @AccessLog.logfile_format)
      assert_equal(1, @logfile_format_call)
    end

    def test_set_logfile_format
      init_context
      @AccessLog.logfile = { :params => @logfile }
      @AccessLog.logfile_format = 'bar'
      assert_equal(1, @set_logfile_format_call)
      assert_equal('bar', @set_logfile_format_value)
    end

    def test_del_logfile
      init_context
      @AccessLog.logfile = { :pos => 0 }
      @AccessLog.del_logfile
      assert_equal(1, @del_logfile_at_call)
      assert_equal(0, @del_logfile_at_pos)
      assert_equal(1, @set_access_log_params_call)
      assert_equal(@access_log_params, @set_access_log_params_value)
    end

    def test_del_logfile_out_of_range
      init_context
      @AccessLog.logfile = { :pos => -1 }
      assert_exception(RuntimeError) {
	@AccessLog.del_logfile
      }
      @AccessLog.logfile = { :pos => 2 }
      assert_exception(RuntimeError) {
	@AccessLog.del_logfile
      }
    end

    def test_write
      init_context
      @AccessLog.write
      assert_equal(0, @del_logfile_at_call)
      assert_equal(1, @set_access_log_params_call)
      assert_equal(@access_log_params, @set_access_log_params_value)
    end
  end
end
