#!/usr/local/bin/ruby
# $Id: test_AddAccessLog.rb,v 1.6 2005/01/16 12:32:31 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'wpm'

module TestRucy
  class TestAddAccessLog < RUNIT::TestCase
    def setup
      # for WPM::Page class
      @driver_call = 0

      @page = Forwarder.new(self)
      class << @page
	def_delegator :__getobj__, :driver
      end

      # for WPM::MessageManipulator class
      @redirect_call = 0
      @redirect_page_name = nil

      @driver = Forwarder.new(self)
      class << @driver
	def_delegator :__getobj__, :redirect
      end

      # for Rucy::Control class
      @access_log_params_call = 0
      @set_access_log_params_call = 0
      @set_access_log_params_value = nil

      @control = Forwarder.new(self)
      class << @control
	def_delegator :__getobj__, :access_log_params
	def_delegator :__getobj__, :set_access_log_params
      end

      # for Rucy::AccessLogParams class
      @modified_count_call = 0
      @set_modified_count_call = 0
      @set_modified_count_value = nil
      @add_logfile_call = 0
      @logfiles_call = 0

      @access_log_params = Forwarder.new(self)
      class << @access_log_params
	def_delegator :__getobj__, :modified_count
	def_delegator :__getobj__, :modified_count=
	def_delegator :__getobj__, :add_logfile
	def_delegator :__getobj__, :logfiles
      end

      # for Rucy::AccessLogParams::LogFile class
      @path_call = 0
      @set_path_call = 0
      @set_path_value = nil
      @format_call = 0
      @set_format_call = 0
      @set_format_value = nil

      @logfile = Forwarder.new(self)
      class << @logfile
	def_delegator :__getobj__, :path
	def_delegator :__getobj__, :path=
	def_delegator :__getobj__, :format
	def_delegator :__getobj__, :format=
      end
      @logfile_list = [ @logfile ]

      # setup WPM::PageContext class
      loader = WPM::Loader.new('../control/AddAccessLog/AddAccessLog.rb')
      option = { :control => @control }
      @AddAccessLog = loader.const_get('AddAccessLog').new(@page, option)
      assert((@AddAccessLog.is_a? WPM::PageContext))
      @AddAccessLog.init_context

      # test of init
      assert_equal(1, @driver_call)
      assert_equal(1, @access_log_params_call)
      assert_equal(1, @add_logfile_call)
      assert_equal(1, @logfiles_call)
    end

    # for WPM::Page class

    def driver
      @driver_call += 1
      @driver
    end

    # for WPM::MessageManipulator class

    def redirect(page_name)
      @redirect_call += 1
      @redirect_page_name = page_name
      nil
    end

    # for Rucy::Control class

    def access_log_params
      @access_log_params_call += 1
      @access_log_params
    end

    def set_access_log_params(params)
      @set_access_log_params_call += 1
      @set_access_log_params_value = params
      nil
    end

    # for Rucy::AccessLogParams class

    def modified_count
      @modified_count_call += 1
      7
    end

    def modified_count=(new_count)
      @set_modified_count_call += 1
      @set_modified_count_value = new_count
    end

    def add_logfile
      @add_logfile_call += 1
      nil
    end

    def logfiles
      @logfiles_call += 1
      @logfile_list.dup
    end

    # for Rucy::AccessLogParams::LogFile class

    def path
      @path_call += 1
      'access.log'
    end

    def path=(new_path)
      @set_path_call += 1
      @set_path_value = new_path
    end

    def format
      @format_call += 1
      'foo'
    end

    def format=(new_format)
      @set_format_call += 1
      @set_format_value = new_format
    end

    # test

    def test_modified_count
      assert_equal(7, @AddAccessLog.modified_count)
      assert_equal(1, @modified_count_call)
      @AddAccessLog.modified_count = 8
      assert_equal(1, @set_modified_count_call)
      assert_equal(8, @set_modified_count_value)
    end

    def test_index
      assert_equal(1, @AddAccessLog.index)
    end

    def test_path
      assert_equal('access.log', @AddAccessLog.path)
      assert_equal(1, @path_call)
    end

    def test_set_path
      @AddAccessLog.path = 'access2.log'
      assert_equal(1, @set_path_call)
      assert_equal('access2.log', @set_path_value)
    end

    def test_format
      assert_equal('foo', @AddAccessLog.format)
      assert_equal(1, @format_call)
    end

    def test_set_format
      @AddAccessLog.format = 'bar'
      assert_equal(1, @set_format_call)
      assert_equal('bar', @set_format_value)
    end

    def test_write
      @AddAccessLog.write
      assert_equal(1, @set_access_log_params_call)
      assert_equal(@access_log_params, @set_access_log_params_value)
      assert_equal(1, @redirect_call)
      assert_equal('AccessLog', @redirect_page_name)
    end

    def test_cancel
      @AddAccessLog.cancel
      assert_equal(0, @set_access_log_params_call)
      assert_equal(1, @redirect_call)
      assert_equal('AccessLog', @redirect_page_name)
    end
  end
end
