#!/usr/local/bin/ruby
# $Id: test_AddLogFile.rb,v 1.6 2005/01/16 12:32:33 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'wpm'

module TestRucy
  class TestAddLogFile < RUNIT::TestCase
    def setup
      # for WPM::Page class
      @driver_call = 0

      @page = Forwarder.new(self)
      class << @page
	def_delegator :__getobj__, :driver
      end

      # for WPM::MessageManipulator class
      @redirect_call = 0
      @redirect_page_name = nil

      @driver = Forwarder.new(self)
      class << @driver
	def_delegator :__getobj__, :redirect
      end

      # for Rucy::Control class
      @logging_params_call = 0
      @set_logging_params_call = 0
      @set_logging_params_value = nil

      @control = Forwarder.new(self)
      class << @control
	def_delegator :__getobj__, :logging_params
	def_delegator :__getobj__, :set_logging_params
      end

      # for Rucy::LoggingParams class
      @modified_count_call = 0
      @set_modified_count_call = 0
      @set_modified_count_value = nil
      @add_logfile_call = 0
      @logfiles_call = 0

      @logging_params = Forwarder.new(self)
      class << @logging_params
	def_delegator :__getobj__, :modified_count
	def_delegator :__getobj__, :modified_count=
	def_delegator :__getobj__, :add_logfile
	def_delegator :__getobj__, :logfiles
      end

      # for Rucy::LoggingParams::LogFile class
      @path_call = 0
      @set_path_call = 0
      @set_path_value = nil
      @logging_level_call = 0
      @set_logging_level_call = 0
      @set_logging_level_value = nil

      @logfile = Forwarder.new(self)
      class << @logfile
	def_delegator :__getobj__, :path
	def_delegator :__getobj__, :path=
	def_delegator :__getobj__, :logging_level
	def_delegator :__getobj__, :logging_level=
      end
      @logfile_list = [ @logfile ]

      # setup WPM::PageContext class
      loader = WPM::Loader.new('../control/AddLogFile/AddLogFile.rb')
      option = { :control => @control }
      @AddLogFile = loader.const_get('AddLogFile').new(@page, option)
      assert((@AddLogFile.is_a? WPM::PageContext))
      @AddLogFile.init_context

      # test of init
      assert_equal(1, @driver_call)
      assert_equal(1, @logging_params_call)
      assert_equal(1, @add_logfile_call)
      assert_equal(1, @logfiles_call)
    end

    # for WPM::Page class

    def driver
      @driver_call += 1
      @driver
    end

    # for WPM::MessageManipulator class

    def redirect(page_name)
      @redirect_call += 1
      @redirect_page_name = page_name
      nil
    end

    # for Rucy::Control class

    def logging_params
      @logging_params_call += 1
      @logging_params
    end

    def set_logging_params(logging_params)
      @set_logging_params_call += 1
      @set_logging_params_value = logging_params
      nil
    end

    # for Rucy::LoggingParams class

    def modified_count
      @modified_count_call += 1
      7
    end

    def modified_count=(new_count)
      @set_modified_count_call += 1
      @set_modified_count_value = new_count
    end

    def add_logfile
      @add_logfile_call += 1
      nil
    end

    def logfiles
      @logfiles_call += 1
      @logfile_list.dup
    end

    # for Rucy::LoggingParams::LogFile class

    def path
      @path_call += 1
      'rucy.log'
    end

    def path=(new_path)
      @set_path_call += 1
      @set_path_value = new_path
    end

    def logging_level
      @logging_level_call += 1
      'notice'
    end

    def logging_level=(new_level)
      @set_logging_level_call += 1
      @set_logging_level_value = new_level
    end

    # test

    def test_modified_count
      assert_equal(7, @AddLogFile.modified_count)
      assert_equal(1, @modified_count_call)
      @AddLogFile.modified_count = 8
      assert_equal(1, @set_modified_count_call)
      assert_equal(8, @set_modified_count_value)
    end

    def test_index
      assert_equal(1, @AddLogFile.index)
    end

    def test_path
      assert_equal('rucy.log', @AddLogFile.path)
      assert_equal(1, @path_call)
    end

    def test_set_path
      @AddLogFile.path = 'server.log'
      assert_equal(1, @set_path_call)
      assert_equal('server.log', @set_path_value)
    end

    def test_logging_level
      assert_equal('notice', @AddLogFile.logging_level)
      assert_equal(1, @logging_level_call)
    end

    def test_set_logging_level
      @AddLogFile.logging_level = 'info'
      assert_equal(1, @set_logging_level_call)
      assert_equal('info', @set_logging_level_value)
    end

    def test_write
      @AddLogFile.write
      assert_equal(1, @set_logging_params_call)
      assert_equal(@logging_params, @set_logging_params_value)
      assert_equal(1, @redirect_call)
      assert_equal('Logging', @redirect_page_name)
    end

    def test_cancel
      @AddLogFile.cancel
      assert_equal(0, @set_logging_params_call)
      assert_equal(1, @redirect_call)
      assert_equal('Logging', @redirect_page_name)
    end
  end
end
