#!/usr/local/bin/ruby
# $Id: test_Logging.rb,v 1.8 2005/01/16 11:08:17 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'wpm'

module TestRucy
  class TestLogging < RUNIT::TestCase
    def setup
      # for WPM::Page class
      @page = Forwarder.new(self)

      # for Rucy::Control class
      @logging_params_call = 0
      @set_logging_params_call = 0
      @set_logging_params_value = nil

      @control = Forwarder.new(self)
      class << @control
	def_delegator :__getobj__, :logging_params
	def_delegator :__getobj__, :set_logging_params
      end

      # for Rucy::LoggingParams class
      @modified_count_call = 0
      @set_modified_count_call = 0
      @set_modified_count_value = nil
      @stdout_logging_level_call = 0
      @set_stdout_logging_level_call = 0
      @set_stdout_logging_level_value = nil
      @logfiles_call = 0
      @del_logfile_at_call = 0
      @del_logfile_at_pos = nil

      @logging_params = Forwarder.new(self)
      class << @logging_params
	def_delegator :__getobj__, :modified_count
	def_delegator :__getobj__, :modified_count=
	def_delegator :__getobj__, :stdout_logging_level
	def_delegator :__getobj__, :stdout_logging_level=
	def_delegator :__getobj__, :logfiles
	def_delegator :__getobj__, :del_logfile_at
      end

      # for Rucy::LoggingParams::LogFile class
      @path_call = 0
      @set_path_call = 0
      @set_path_value = nil
      @logging_level_call = 0
      @set_logging_level_call = 0
      @set_logging_level_value = nil

      @logfile = Forwarder.new(self)
      class << @logfile
	def_delegator :__getobj__, :path
	def_delegator :__getobj__, :path=
	def_delegator :__getobj__, :logging_level
	def_delegator :__getobj__, :logging_level=
      end
      @logfile2 = Object.new	# dummy
      @logfiles_list = [ @logfile, @logfile2 ]

      # setup WPM::PageContext class
      loader = WPM::Loader.new('../control/Logging/Logging.rb')
      option = { :control => @control }
      @Logging = loader.const_get('Logging').new(@page, option)
      assert((@Logging.is_a? WPM::PageContext))
    end

    # for Rucy::Control class

    def logging_params
      @logging_params_call += 1
      @logging_params
    end

    def set_logging_params(logging_params)
      @set_logging_params_call += 1
      @set_logging_params_value = logging_params
      nil
    end

    # for Rucy::LoggingParams class

    def modified_count
      @modified_count_call += 1
      7
    end

    def modified_count=(new_count)
      @set_modified_count_call += 1
      @set_modified_count_value = new_count
    end

    def stdout_logging_level
      @stdout_logging_level_call += 1
      'notice'
    end

    def stdout_logging_level=(new_level)
      @set_stdout_logging_level_call += 1
      @set_stdout_logging_level_value = new_level
    end

    def logfiles
      @logfiles_call += 1
      @logfiles_list.dup
    end

    def del_logfile_at(pos)
      @del_logfile_at_call += 1
      @del_logfile_at_pos = pos
      nil
    end

    # for Rucy::LoggingParams::LogFile class

    def path
      @path_call += 1
      'rucy.log'
    end

    def path=(new_path)
      @set_path_call += 1
      @set_path_value = new_path
    end

    def logging_level
      @logging_level_call += 1
      'notice'
    end

    def logging_level=(new_level)
      @set_logging_level_call += 1
      @set_logging_level_value = new_level
    end

    # test

    def init_context
      @Logging.init_context
      assert_equal(1, @logging_params_call)
      assert_equal(0, @logfiles_call)
    end

    def test_modified_count
      init_context
      assert_equal(7, @Logging.modified_count)
      assert_equal(1, @modified_count_call)
      @Logging.modified_count = 8
      assert_equal(1, @set_modified_count_call)
      assert_equal(8, @set_modified_count_value)
    end

    def test_stdout_logging_level
      init_context
      assert_equal('notice', @Logging.stdout_logging_level)
      assert_equal(1, @stdout_logging_level_call)
    end

    def test_set_stdout_logging_level
      init_context
      @Logging.stdout_logging_level = 'info'
      assert_equal(1, @set_stdout_logging_level_call)
      assert_equal('info', @set_stdout_logging_level_value)
    end

    def test_has_logfiles_true
      init_context
      assert_equal(true, @Logging.has_logfiles?)
      assert_equal(1, @logfiles_call)
    end

    def test_has_logfiles_false
      @logfiles_list.clear
      init_context
      assert_equal(false, @Logging.has_logfiles?)
      assert_equal(1, @logfiles_call)
    end

    def test_index
      @Logging.index = 1
      assert_equal(1, @Logging.index)
    end

    def test_logfile_list
      init_context
      assert_equal([ { :pos => 0, :params => @logfile },
		     { :pos => 1, :params => @logfile2 }
		   ], @Logging.logfile_list)
      assert_equal(1, @logfiles_call)
    end

    def test_logfile_path
      init_context
      @Logging.logfile = { :params => @logfile }
      assert_equal('rucy.log', @Logging.logfile_path)
      assert_equal(1, @path_call)
    end

    def test_set_logfile_path
      init_context
      @Logging.logfile = { :params => @logfile }
      @Logging.logfile_path = 'server.log'
      assert_equal(1, @set_path_call)
      assert_equal('server.log', @set_path_value)
    end

    def test_logfile_logging_level
      init_context
      @Logging.logfile = { :params => @logfile }
      assert_equal('notice', @Logging.logfile_logging_level)
      assert_equal(1, @logging_level_call)
    end

    def test_logfile_logging_level
      init_context
      @Logging.logfile = { :params => @logfile }
      @Logging.logfile_logging_level = 'info'
      assert_equal(1, @set_logging_level_call)
      assert_equal('info', @set_logging_level_value)
    end

    def test_del_logfile
      init_context
      @Logging.logfile = { :pos => 0 }
      @Logging.del_logfile
      assert_equal(1, @del_logfile_at_call)
      assert_equal(0, @del_logfile_at_pos)
      assert_equal(1, @set_logging_params_call)
      assert_equal(@logging_params, @set_logging_params_value)
    end

    def test_del_logfile_out_of_range
      init_context
      @Logging.logfile = { :pos => -1 }
      assert_exception(RuntimeError) {
	@Logging.del_logfile
      }
      @Logging.logfile = { :pos => 2 }
      assert_exception(RuntimeError) {
	@Logging.del_logfile
      }
    end

    def test_write
      init_context
      @Logging.write
      assert_equal(0, @del_logfile_at_call)
      assert_equal(1, @set_logging_params_call)
      assert_equal(@logging_params, @set_logging_params_value)
    end
  end
end
