#!/usr/local/bin/ruby
# $Id: test_ServerError.rb,v 1.1 2005/02/05 01:41:23 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'wpm'

module TestRucy
  class TestServerError < RUNIT::TestCase
    def setup
      # for WPM::Page class
      @page = Forwarder.new(self)

      # for Rucy::Control class
      @setup_load_errors_call = 0
      @setup_load_error_list = nil
      @setup_logging_errors_call = 0
      @setup_logging_error_list = nil
      @setup_alias_errors_call = 0
      @setup_alias_error_list = nil
      @setup_document_errors_call = 0
      @setup_document_error_list = nil
      @setup_filter_errors_call = 0
      @setup_filter_error_list = nil

      @control = Forwarder.new(self)
      class << @control
	def_delegator :__getobj__, :setup_load_errors
	def_delegator :__getobj__, :setup_logging_errors
	def_delegator :__getobj__, :setup_alias_errors
	def_delegator :__getobj__, :setup_document_errors
	def_delegator :__getobj__, :setup_filter_errors
      end

      # for Rucy::RuntimeErrorSample class
      @timestamp_call = 0
      @error_message_call = 0
      @error_type_call = 0

      @error_sample = Forwarder.new(self)
      class << @error_sample
	def_delegator :__getobj__, :timestamp
	def_delegator :__getobj__, :error_message
	def_delegator :__getobj__, :error_type
      end
      @dummy = Object.new
      @setup_load_error_list = [ @error_sample, @dummy, @dummy, @dummy, @dummy ]
      @setup_logging_error_list = [ @dummy, @error_sample, @dummy, @dummy, @dummy ]
      @setup_alias_error_list = [ @dummy, @dummy, @error_sample, @dummy, @dummy ]
      @setup_document_error_list = [ @dummy, @dummy, @dummy, @error_sample, @dummy ]
      @setup_filter_error_list = [ @dummy, @dummy, @dummy, @dummy, @error_sample ]

      # setup WPM::PageContext class
      loader = WPM::Loader.new('../control/ServerError/ServerError.rb')
      option = { :control => @control }
      @ServerError = loader.const_get('ServerError').new(@page, option)
      @ServerError.init_context
    end

    # for Rucy::Control class

    def setup_load_errors
      @setup_load_errors_call += 1
      @setup_load_error_list
    end

    def setup_logging_errors
      @setup_logging_errors_call += 1
      @setup_logging_error_list
    end

    def setup_alias_errors
      @setup_alias_errors_call += 1
      @setup_alias_error_list
    end

    def setup_document_errors
      @setup_document_errors_call += 1
      @setup_document_error_list
    end

    def setup_filter_errors
      @setup_filter_errors_call += 1
      @setup_filter_error_list
    end

    # for Rucy::RuntimeErrorSample class

    def timestamp
      @timestamp_call += 1
      Time.mktime(2005, 2, 3, 0, 57, 13)
    end

    def error_message
      @error_message_call += 1
      'foo'
    end

    def error_type
      @error_type_call += 1
      :TestErrorType
    end

    # test

    def test_setup_load_errors
      assert_equal([ @error_sample, @dummy, @dummy, @dummy, @dummy ],
		   @ServerError.setup_load_errors)
      assert_equal(1, @setup_load_errors_call)
    end

    def test_setup_logging_errors
      assert_equal([ @dummy, @error_sample, @dummy, @dummy, @dummy ],
		   @ServerError.setup_logging_errors)
      assert_equal(1, @setup_logging_errors_call)
    end

    def test_setup_alias_errors
      assert_equal([ @dummy, @dummy, @error_sample, @dummy, @dummy ],
		   @ServerError.setup_alias_errors)
      assert_equal(1, @setup_alias_errors_call)
    end

    def test_setup_document_errors
      assert_equal([ @dummy, @dummy, @dummy, @error_sample, @dummy ],
		   @ServerError.setup_document_errors)
      assert_equal(1, @setup_document_errors_call)
    end

    def test_setup_filter_errors
      assert_equal([ @dummy, @dummy, @dummy, @dummy, @error_sample ],
		   @ServerError.setup_filter_errors)
      assert_equal(1, @setup_filter_errors_call)
    end

    def test_has_load_errors_true
      assert_equal(true, @ServerError.has_load_errors?)
      assert_equal(1, @setup_load_errors_call)
    end

    def test_has_load_errors_false
      @setup_load_error_list = []
      assert_equal(false, @ServerError.has_load_errors?)
      assert_equal(1, @setup_load_errors_call)
    end

    def test_has_logging_errors_true
      assert_equal(true, @ServerError.has_logging_errors?)
      assert_equal(1, @setup_logging_errors_call)
    end

    def test_has_logging_errors_false
      @setup_logging_error_list = []
      assert_equal(false, @ServerError.has_logging_errors?)
      assert_equal(1, @setup_logging_errors_call)
    end

    def test_has_alias_errors_true
      assert_equal(true, @ServerError.has_alias_errors?)
      assert_equal(1, @setup_alias_errors_call)
    end

    def test_has_alias_errors_false
      @setup_alias_error_list = []
      assert_equal(false, @ServerError.has_alias_errors?)
      assert_equal(1, @setup_alias_errors_call)
    end

    def test_has_document_errors_true
      assert_equal(true, @ServerError.has_document_errors?)
      assert_equal(1, @setup_document_errors_call)
    end

    def test_has_document_errors_false
      @setup_document_error_list = []
      assert_equal(false, @ServerError.has_document_errors?)
      assert_equal(1, @setup_document_errors_call)
    end

    def test_has_filter_errors_true
      assert_equal(true, @ServerError.has_filter_errors?)
      assert_equal(1, @setup_filter_errors_call)
    end

    def test_has_filter_errors_false
      @setup_filter_error_list = []
      assert_equal(false, @ServerError.has_filter_errors?)
      assert_equal(1, @setup_filter_errors_call)
    end

    def test_timestamp
      @ServerError.error_sample = @error_sample
      assert_equal(Time.mktime(2005, 2, 3, 0, 57, 13), @ServerError.timestamp)
      assert_equal(1, @timestamp_call)
    end

    def test_error_message
      @ServerError.error_sample = @error_sample
      assert_equal('foo', @ServerError.error_message)
      assert_equal(1, @error_message_call)
    end

    def test_error_type
      @ServerError.error_sample = @error_sample
      assert_equal(:TestErrorType, @ServerError.error_type)
      assert_equal(1, @error_type_call)
    end

    def test_load_error_query
      @ServerError.error_sample = @error_sample
      assert_equal({ 'type' => 'load', 'id' => '0' }, @ServerError.load_error_query)
      assert_equal(1, @setup_load_errors_call)
    end

    def test_logging_error_query
      @ServerError.error_sample = @error_sample
      assert_equal({ 'type' => 'logging', 'id' => '1' }, @ServerError.logging_error_query)
      assert_equal(1, @setup_logging_errors_call)
    end

    def test_alias_error_query
      @ServerError.error_sample = @error_sample
      assert_equal({ 'type' => 'alias', 'id' => '2' }, @ServerError.alias_error_query)
      assert_equal(1, @setup_alias_errors_call)
    end

    def test_document_error_query
      @ServerError.error_sample = @error_sample
      assert_equal({ 'type' => 'document', 'id' => '3' }, @ServerError.document_error_query)
      assert_equal(1, @setup_document_errors_call)
    end

    def test_filter_error_queries
      @ServerError.error_sample = @error_sample
      assert_equal({ 'type' => 'filter', 'id' => '4' }, @ServerError.filter_error_query)
      assert_equal(1, @setup_filter_errors_call)
    end
  end
end
