#!/usr/local/bin/ruby
# $Id: test_ServerErrorInfo.rb,v 1.1 2005/02/05 14:35:45 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'wpm'

module TestRucy
  class TestServerErrorInfo < RUNIT::TestCase
    def setup
      # for WPM::Page class
      @driver_call = 0

      @page = Forwarder.new(self)
      class << @page
	def_delegator :__getobj__, :driver
      end

      # for WPM::MessageManipulator class
      @params_call = 0
      @params = Hash.new

      @driver = Forwarder.new(self)
      class << @driver
	def_delegator :__getobj__, :params
      end

      # for Rucy::Control class
      @setup_load_errors_call = 0
      @setup_load_error_list = nil
      @setup_logging_errors_call = 0
      @setup_logging_error = nil
      @setup_alias_errors_call = 0
      @setup_alias_error_list = nil
      @setup_document_errors_call = 0
      @setup_document_error_list = nil
      @setup_filter_errors_call = 0
      @setup_filter_error_list = nil

      @control = Forwarder.new(self)
      class << @control
	def_delegator :__getobj__, :setup_load_errors
	def_delegator :__getobj__, :setup_logging_errors
	def_delegator :__getobj__, :setup_alias_errors
	def_delegator :__getobj__, :setup_document_errors
	def_delegator :__getobj__, :setup_filter_errors
      end

      # for Rucy::RuntimeErrorSample class (superclass of error samples)
      @error_message_call = 0
      @error_type_call = 0
      @backtrace_call = 0
      @backtrace = [ 'foo', 'bar' ]
      @timestamp_call = 0

      # for Rucy::LoadErrorSample class
      @mod_path_call = 0

      @load_error_sample = Forwarder.new(self)
      class << @load_error_sample
	def_delegator :__getobj__, :error_message
	def_delegator :__getobj__, :error_type
	def_delegator :__getobj__, :timestamp
	def_delegator :__getobj__, :backtrace
	def_delegator :__getobj__, :mod_path
      end

      # for Rucy::LoggingErrorSample class
      @log_path_call = 0

      @logging_error_sample = Forwarder.new(self)
      class << @logging_error_sample
	def_delegator :__getobj__, :error_message
	def_delegator :__getobj__, :error_type
	def_delegator :__getobj__, :timestamp
	def_delegator :__getobj__, :backtrace
	def_delegator :__getobj__, :log_path
      end

      # for Rucy::AliasErrorSample class
      @alias_path_call = 0
      @orig_path_call = 0
      @alias_virtual_host_call = 0

      @alias_error_sample = Forwarder.new(self)
      class << @alias_error_sample
	def_delegator :__getobj__, :error_message
	def_delegator :__getobj__, :error_type
	def_delegator :__getobj__, :timestamp
	def_delegator :__getobj__, :backtrace
	def_delegator :__getobj__, :alias_path
	def_delegator :__getobj__, :orig_path
	def_delegator :__getobj__, :alias_virtual_host, :virtual_host
      end

      # for Rucy::DocumentErrorSample class
      @document_call = 0
      @document_arguments_call = 0
      @mount_path_call = 0
      @mount_mask_call = 0
      @document_virtual_host_call = 0

      @document_error_sample = Forwarder.new(self)
      class << @document_error_sample
	def_delegator :__getobj__, :error_message
	def_delegator :__getobj__, :error_type
	def_delegator :__getobj__, :timestamp
	def_delegator :__getobj__, :backtrace
	def_delegator :__getobj__, :document
	def_delegator :__getobj__, :document_arguments, :arguments
	def_delegator :__getobj__, :mount_path
	def_delegator :__getobj__, :mount_mask
	def_delegator :__getobj__, :document_virtual_host, :virtual_host
      end

      # for Rucy::FilterErrorSample class
      @filter_call = 0
      @filter_arguments_call = 0
      @attach_path_call = 0
      @attach_mask_call = 0
      @filter_virtual_host_call = 0

      @filter_error_sample = Forwarder.new(self)
      class << @filter_error_sample
	def_delegator :__getobj__, :error_message
	def_delegator :__getobj__, :error_type
	def_delegator :__getobj__, :timestamp
	def_delegator :__getobj__, :backtrace
	def_delegator :__getobj__, :filter
	def_delegator :__getobj__, :filter_arguments, :arguments
	def_delegator :__getobj__, :attach_path
	def_delegator :__getobj__, :attach_mask
	def_delegator :__getobj__, :filter_virtual_host, :virtual_host
      end

      # setup WPM::PageContext class
      loader = WPM::Loader.new('../control/ServerErrorInfo/ServerErrorInfo.rb')
      option = { :control => @control }
      @ServerErrorInfo = loader.const_get('ServerErrorInfo').new(@page, option)
    end

    # for WPM::Page class

    def driver
      @driver_call += 1
      @driver
    end

    # for WPM::MessageManipulator class

    def params
      @params_call += 1
      @params
    end

    # for Rucy::Control class

    def setup_load_errors
      @setup_load_errors_call += 1
      @setup_load_error_list
    end

    def setup_logging_errors
      @setup_logging_errors_call += 1
      @setup_logging_error_list
    end

    def setup_alias_errors
      @setup_alias_errors_call += 1
      @setup_alias_error_list
    end

    def setup_document_errors
      @setup_document_errors_call += 1
      @setup_document_error_list
    end

    def setup_filter_errors
      @setup_filter_errors_call += 1
      @setup_filter_error_list
    end

    # for Rucy::RuntimeErrorSample class

    def error_message
      @error_message_call += 1
      'foo'
    end

    def error_type
      @error_type_call += 1
      :TestErrorType
    end

    def backtrace
      @backtrace_call += 1
      @backtrace
    end

    def timestamp
      @timestamp_call += 1
      Time.mktime(2005, 2, 3, 0, 57, 13)
    end

    # for Rucy::LoadErrorSample class

    def mod_path
      @mod_path_call += 1
      'mod_docs/foo.rb'
    end

    # for Rucy::LoggingErrorSample class

    def log_path
      @log_path_call += 1
      'foo.log'
    end

    # for Rucy::AliasErrorSample class

    def alias_path
      @alias_path_call += 1
      '/foo'
    end

    def orig_path
      @orig_path_call += 1
      '/bar'
    end

    def alias_virtual_host
      @alias_virtual_host_call += 1
      'www.foo.net'
    end

    # for Rucy::DocumentErrorSample class

    def document
      @document_call += 1
      'Foo'
    end

    def document_arguments
      @document_arguments_call += 1
      []
    end

    def mount_path
      @mount_path_call += 1
      '/foo'
    end

    def mount_mask
      @mount_mask_call += 1
      ''
    end

    def document_virtual_host
      @document_virtual_host_call += 1
      'www.foo.net'
    end

    # for Rucy::FilterErrorSample class

    def filter
      @filter_call += 1
      'Foo'
    end

    def filter_arguments
      @filter_arguments_call += 1
      []
    end

    def attach_path
      @attach_path_call += 1
      '/foo'
    end

    def attach_mask
      @attach_mask_call += 1
      ''
    end

    def filter_virtual_host
      @filter_virtual_host_call += 1
      'www.foo.net'
    end

    # test

    def init_ServerErrorInfo
      @ServerErrorInfo.init_context
      assert_equal(1, @driver_call)
      assert_equal(1, @params_call)
    end

    def test_load_error
      @params['type'] = 'load'
      @params['id'] = '0'
      @setup_load_error_list = [ @load_error_sample ]
      init_ServerErrorInfo
      assert_equal(1, @setup_load_errors_call)
      assert_equal(true, @ServerErrorInfo.load_error?)
      assert_equal(false, @ServerErrorInfo.logging_error?)
      assert_equal(false, @ServerErrorInfo.alias_error?)
      assert_equal(false, @ServerErrorInfo.document_error?)
      assert_equal(false, @ServerErrorInfo.filter_error?)

      # for Rucy::RuntimeErrorSample class
      assert_equal('foo', @ServerErrorInfo.error_message)
      assert_equal(1, @error_message_call)
      assert_equal(:TestErrorType, @ServerErrorInfo.error_type)
      assert_equal(1, @error_type_call)
      assert_equal(Time.mktime(2005, 2, 3, 0, 57, 13), @ServerErrorInfo.timestamp)
      assert_equal(1, @timestamp_call)
      assert_equal([ 'foo', 'bar' ], @ServerErrorInfo.backtrace)
      assert_equal(1, @backtrace_call)

      # for Rucy::LoadErrorSample class
      assert_equal('mod_docs/foo.rb', @ServerErrorInfo.mod_path)
      assert_equal(1, @mod_path_call)
    end

    def test_logging_error
      @params['type'] = 'logging'
      @params['id'] = '1'
      @setup_logging_error_list = [ nil, @logging_error_sample ]
      init_ServerErrorInfo
      assert_equal(1, @setup_logging_errors_call)
      assert_equal(false, @ServerErrorInfo.load_error?)
      assert_equal(true, @ServerErrorInfo.logging_error?)
      assert_equal(false, @ServerErrorInfo.alias_error?)
      assert_equal(false, @ServerErrorInfo.document_error?)
      assert_equal(false, @ServerErrorInfo.filter_error?)

      # for Rucy::RuntimeErrorSample class
      assert_equal('foo', @ServerErrorInfo.error_message)
      assert_equal(1, @error_message_call)
      assert_equal(:TestErrorType, @ServerErrorInfo.error_type)
      assert_equal(1, @error_type_call)
      assert_equal(Time.mktime(2005, 2, 3, 0, 57, 13), @ServerErrorInfo.timestamp)
      assert_equal(1, @timestamp_call)
      assert_equal([ 'foo', 'bar' ], @ServerErrorInfo.backtrace)
      assert_equal(1, @backtrace_call)

      # for Rucy::LoggingErrorSample class
      assert_equal('foo.log', @ServerErrorInfo.log_path)
      assert_equal(1, @log_path_call)
    end

    def test_alias_error
      @params['type'] = 'alias'
      @params['id'] = '2'
      @setup_alias_error_list = [ nil, nil, @alias_error_sample ]
      init_ServerErrorInfo
      assert_equal(1, @setup_alias_errors_call)
      assert_equal(false, @ServerErrorInfo.load_error?)
      assert_equal(false, @ServerErrorInfo.logging_error?)
      assert_equal(true, @ServerErrorInfo.alias_error?)
      assert_equal(false, @ServerErrorInfo.document_error?)
      assert_equal(false, @ServerErrorInfo.filter_error?)

      # for Rucy::RuntimeErrorSample class
      assert_equal('foo', @ServerErrorInfo.error_message)
      assert_equal(1, @error_message_call)
      assert_equal(:TestErrorType, @ServerErrorInfo.error_type)
      assert_equal(1, @error_type_call)
      assert_equal(Time.mktime(2005, 2, 3, 0, 57, 13), @ServerErrorInfo.timestamp)
      assert_equal(1, @timestamp_call)
      assert_equal([ 'foo', 'bar' ], @ServerErrorInfo.backtrace)
      assert_equal(1, @backtrace_call)

      # for Rucy::AliasErrorSample class
      assert_equal('/foo', @ServerErrorInfo.alias_path)
      assert_equal(1, @alias_path_call)
      assert_equal('/bar', @ServerErrorInfo.orig_path)
      assert_equal(1, @orig_path_call)
      assert_equal('www.foo.net', @ServerErrorInfo.virtual_host)
      assert_equal(1, @alias_virtual_host_call)
    end

    def test_document_error
      @params['type'] = 'document'
      @params['id'] = '3'
      @setup_document_error_list = [ nil, nil, nil, @document_error_sample ]
      init_ServerErrorInfo
      assert_equal(1, @setup_document_errors_call)
      assert_equal(false, @ServerErrorInfo.load_error?)
      assert_equal(false, @ServerErrorInfo.logging_error?)
      assert_equal(false, @ServerErrorInfo.alias_error?)
      assert_equal(true, @ServerErrorInfo.document_error?)
      assert_equal(false, @ServerErrorInfo.filter_error?)

      # for Rucy::RuntimeErrorSample class
      assert_equal('foo', @ServerErrorInfo.error_message)
      assert_equal(1, @error_message_call)
      assert_equal(:TestErrorType, @ServerErrorInfo.error_type)
      assert_equal(1, @error_type_call)
      assert_equal(Time.mktime(2005, 2, 3, 0, 57, 13), @ServerErrorInfo.timestamp)
      assert_equal(1, @timestamp_call)
      assert_equal([ 'foo', 'bar' ], @ServerErrorInfo.backtrace)
      assert_equal(1, @backtrace_call)

      # for Rucy::DocumentErrorSample class
      assert_equal('Foo', @ServerErrorInfo.document)
      assert_equal(1, @document_call)
      assert_equal([], @ServerErrorInfo.arguments)
      assert_equal(1, @document_arguments_call)
      assert_equal('/foo', @ServerErrorInfo.mount_path)
      assert_equal(1, @mount_path_call)
      assert_equal('', @ServerErrorInfo.mount_mask)
      assert_equal(1, @mount_mask_call)
      assert_equal('www.foo.net', @ServerErrorInfo.virtual_host)
      assert_equal(1, @document_virtual_host_call)
    end

    def test_filter_error
      @params['type'] = 'filter'
      @params['id'] = '4'
      @setup_filter_error_list = [ nil, nil, nil, nil, @filter_error_sample ]
      init_ServerErrorInfo
      assert_equal(1, @setup_filter_errors_call)
      assert_equal(false, @ServerErrorInfo.load_error?)
      assert_equal(false, @ServerErrorInfo.logging_error?)
      assert_equal(false, @ServerErrorInfo.alias_error?)
      assert_equal(false, @ServerErrorInfo.document_error?)
      assert_equal(true, @ServerErrorInfo.filter_error?)

      # for Rucy::RuntimeErrorSample class
      assert_equal('foo', @ServerErrorInfo.error_message)
      assert_equal(1, @error_message_call)
      assert_equal(:TestErrorType, @ServerErrorInfo.error_type)
      assert_equal(1, @error_type_call)
      assert_equal(Time.mktime(2005, 2, 3, 0, 57, 13), @ServerErrorInfo.timestamp)
      assert_equal(1, @timestamp_call)
      assert_equal([ 'foo', 'bar' ], @ServerErrorInfo.backtrace)
      assert_equal(1, @backtrace_call)

      # for Rucy::FilterErrorSample class
      assert_equal('Foo', @ServerErrorInfo.filter)
      assert_equal(1, @filter_call)
      assert_equal([], @ServerErrorInfo.arguments)
      assert_equal(1, @filter_arguments_call)
      assert_equal('/foo', @ServerErrorInfo.attach_path)
      assert_equal(1, @attach_path_call)
      assert_equal('', @ServerErrorInfo.attach_mask)
      assert_equal(1, @attach_mask_call)
      assert_equal('www.foo.net', @ServerErrorInfo.virtual_host)
      assert_equal(1, @filter_virtual_host_call)
    end

    def test_backtrace
      @params['type'] = 'load'
      @params['id'] = '0'
      @setup_load_error_list = [ @load_error_sample ]
      init_ServerErrorInfo
      assert_equal(true, @ServerErrorInfo.has_backtrace?)
      assert_equal(1, @backtrace_call)
      @backtrace = nil
      assert_equal(false, @ServerErrorInfo.has_backtrace?)
      assert_equal(2, @backtrace_call)
    end

    def test_reload_query
      @params['type'] = 'load'
      @params['id'] = '0'
      @setup_load_error_list = [ @load_error_sample ]
      init_ServerErrorInfo
      assert_equal({ 'type' => 'load', 'id' => '0' }, @ServerErrorInfo.reload_query)
    end

    def test_for_backtrace_list
      @params['type'] = 'load'
      @params['id'] = '0'
      @setup_load_error_list = [ @load_error_sample ]
      init_ServerErrorInfo
      @ServerErrorInfo.frame_info = 'foo'
      assert_equal('foo', @ServerErrorInfo.frame_info)
    end

    def test_for_argument_list
      @params['type'] = 'load'
      @params['id'] = '0'
      @setup_load_error_list = [ @load_error_sample ]
      init_ServerErrorInfo
      @ServerErrorInfo.arg_value = 'foo'
      assert_equal('foo', @ServerErrorInfo.arg_value)
    end
  end
end
