#!/usr/local/bin/ruby
# $Id: test_SetupAlias.rb,v 1.5 2005/01/01 06:31:23 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'wpm'

module TestRucy
  class TestSetupAlias < RUNIT::TestCase
    def setup
      # for WPM::Page class
      @driver_call = 0

      @page = Forwarder.new(self)
      class << @page
	def_delegator :__getobj__, :driver
      end

      # for WPM::Driver class
      @params_call = 0
      @query_params = { 'pos' => '2' }
      @redirect_call = 0
      @redirect_page_name = nil

      @driver = Forwarder.new(self)
      class << @driver
	def_delegator :__getobj__, :params
	def_delegator :__getobj__, :redirect
      end

      # for Rucy::Control class
      @aliases_call = 0
      @set_aliases_call = 0
      @set_aliases_alias_list = nil

      @control = Forwarder.new(self)
      class << @control
	def_delegator :__getobj__, :aliases
	def_delegator :__getobj__, :set_aliases
      end

      # for Rucy::AliasList class
      @modified_count_call = 0
      @set_modified_count_call = 0
      @set_modified_count_value = nil
      @at_call = 0
      @at_pos = nil
      @length_call = 0
      @add_entry_call = 0

      @alias_list = Forwarder.new(self)
      class << @alias_list
	def_delegator :__getobj__, :modified_count
	def_delegator :__getobj__, :modified_count=
	def_delegator :__getobj__, :at, :[]
	def_delegator :__getobj__, :length
	def_delegator :__getobj__, :add_entry
      end

      # for Rucy::AliasParams class
      @alias_path = nil
      @alias_path_call = 0
      @set_alias_path_call = 0
      @orig_path = nil
      @orig_path_call = 0
      @set_orig_path_call = 0
      @virtual_host = nil
      @virtual_host_call = 0
      @set_virtual_host_call = 0

      @alias_params = Forwarder.new(self)
      class << @alias_params
	def_delegator :__getobj__, :alias_path
	def_delegator :__getobj__, :alias_path=
	def_delegator :__getobj__, :orig_path
	def_delegator :__getobj__, :orig_path=
	def_delegator :__getobj__, :virtual_host
	def_delegator :__getobj__, :virtual_host=
      end

      # setup WPM::PageContext class
      loader = WPM::Loader.new('../control/SetupAlias/SetupAlias.rb')
      option = { :control => @control }
      @SetupAlias = loader.const_get('SetupAlias').new(@page, option)
      assert((@SetupAlias.is_a? WPM::PageContext))
    end

    # for WPM::Page class

    def driver
      @driver_call += 1
      @driver
    end

    # for WPM::Driver class

    def params
      @params_call += 1
      @query_params
    end

    def redirect(page_name)
      @redirect_call += 1
      @redirect_page_name = page_name
      nil
    end

    # for Rucy::Control class

    def aliases
      @aliases_call += 1
      @alias_list
    end

    def set_aliases(alias_list)
      @set_aliases_call += 1
      @set_aliases_alias_list = alias_list
      nil
    end

    # for Rucy::AliasList class

    def modified_count
      @modified_count_call += 1
      7
    end

    def modified_count=(new_count)
      @set_modified_count_call += 1
      @set_modified_count_value = new_count
    end

    def at(pos)
      @at_call += 1
      @at_pos = pos
      @alias_params
    end

    def length
      @length_call += 1
      5
    end

    def add_entry
      @add_entry_call += 1
      nil
    end

    # for Rucy::AliasParams class

    def alias_path
      @alias_path_call += 1
      @alias_path
    end

    def alias_path=(new_path)
      @set_alias_path_call += 1
      @alias_path = new_path
    end

    def orig_path
      @orig_path_call += 1
      @orig_path
    end

    def orig_path=(new_path)
      @set_orig_path_call += 1
      @orig_path = new_path
    end

    def virtual_host
      @virtual_host_call += 1
      @virtual_host
    end

    def virtual_host=(new_host)
      @set_virtual_host_call += 1
      @virtual_host = new_host
    end

    # test

    def test_add_entry
      @query_params['pos'] = '5'
      @SetupAlias.init_context
      assert_equal(1, @driver_call)
      assert_equal(1, @params_call)
      assert_equal(1, @aliases_call)
      assert_equal(1, @length_call)
      assert_equal(1, @add_entry_call)
      assert_equal(1, @at_call)
      assert_equal(5, @at_pos)
    end

    def test_out_of_range
      @query_params['pos'] = '6'
      assert_exception(RuntimeError) {
	@SetupAlias.init_context
      }
    end

    def init_context
      @SetupAlias.init_context
      assert_equal(1, @driver_call)
      assert_equal(1, @params_call)
      assert_equal(1, @aliases_call)
      assert_equal(1, @length_call)
      assert_equal(0, @add_entry_call)
      assert_equal(1, @at_call)
      assert_equal(2, @at_pos)
      nil
    end

    def test_modified_count
      init_context
      assert_equal(7, @SetupAlias.modified_count)
      assert_equal(1, @modified_count_call)
      @SetupAlias.modified_count = 8
      assert_equal(1, @set_modified_count_call)
      assert_equal(8, @set_modified_count_value)
    end

    def test_pos
      init_context
      assert_equal(2, @SetupAlias.pos)
      assert_equal(3, @SetupAlias.index)
    end

    def test_alias_path
      init_context
      @alias_path = '/foo'
      assert_equal('/foo', @SetupAlias.alias_path)
      assert_equal(1, @alias_path_call)
    end

    def test_set_alias_path
      init_context
      @SetupAlias.alias_path = '/foo'
      assert_equal(1, @set_alias_path_call)
      assert_equal('/foo', @alias_path)
    end

    def test_orig_path
      init_context
      @orig_path = '/bar'
      assert_equal('/bar', @SetupAlias.orig_path)
      assert_equal(1, @orig_path_call)
    end

    def test_orig_path
      init_context
      @SetupAlias.orig_path = '/bar'
      assert_equal(1, @set_orig_path_call)
      assert_equal('/bar', @orig_path)
    end

    def test_virtual_host
      init_context
      @virtual_host = 'www.foo.net'
      assert_equal('www.foo.net', @SetupAlias.virtual_host)
      assert_equal(1, @virtual_host_call)
    end

    def test_set_virtual_host
      init_context
      @SetupAlias.virtual_host = 'www.foo.net'
      assert_equal(1, @set_virtual_host_call)
      assert_equal('www.foo.net', @virtual_host)
    end

    def test_reload_query
      init_context
      assert_equal({ 'pos' => '2' }, @SetupAlias.reload_query)
    end

    def test_write
      init_context
      @SetupAlias.write
      assert_equal(1, @set_aliases_call)
      assert_equal(@alias_list, @set_aliases_alias_list)
      assert_equal(1, @redirect_call)
      assert_equal('AliasMap', @redirect_page_name)
    end

    def test_cancel
      init_context
      @SetupAlias.cancel
      assert_equal(0, @set_aliases_call)
      assert_equal(1, @redirect_call)
      assert_equal('AliasMap', @redirect_page_name)
    end
  end
end
