#!/usr/local/bin/ruby
# $Id: test_fork.rb,v 1.2 2005/02/11 16:25:34 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'rucy/rep'
require 'rucy/fork'

module TestRucy
  class TestFork < RUNIT::TestCase
    def setup
      # test repository
      @rep = Rucy::Repository.new

      # for Process module
      @fork_call = 0
      @fork_not_implemented = false
      @fork_child_pid = 1732
      @waitpid_call = 0
      @waitpid_pid = nil

      @Process = Forwarder.new(self)
      class << @Process
	def_delegator :__getobj__, :fork
	def_delegator :__getobj__, :waitpid
      end
      @rep.set_value(:Process_module, @Process)
    end

    # for Process module

    def fork
      @fork_call += 1
      if (@fork_not_implemented) then
	raise NotImplementedError, 'not implemented fork'
      end
      pid = @fork_child_pid
      @fork_child_pid += 1
      pid
    end

    def waitpid(pid)
      @waitpid_call += 1
      @waitpid_pid = pid
      nil
    end

    # test

    def init_fork
      @fork = Rucy::Fork.new(@rep)
    end

    def test_test_forkable_true
      init_fork			# call Rucy::Fork#test_forkable?
      assert_equal(1, @fork_call)
      assert_equal(1, @waitpid_call)
      assert_equal(1732, @waitpid_pid)
      assert_equal(true, @fork.test_forkable?)
      assert_equal(2, @fork_call)
      assert_equal(2, @waitpid_call)
      assert_equal(1733, @waitpid_pid)
    end

    def test_test_forkable_false
      @fork_not_implemented = true
      init_fork			# call Rucy::Fork#test_forkable?
      assert_equal(1, @fork_call)
      assert_equal(0, @waitpid_call)
      assert_equal(false, @fork.test_forkable?)
      assert_equal(2, @fork_call)
      assert_equal(0, @waitpid_call)
    end

    def test_forkable_true
      init_fork			# call Rucy::Fork#test_forkable?
      assert_equal(1, @fork_call)
      assert_equal(1, @waitpid_call)
      assert_equal(1732, @waitpid_pid)
      assert_equal(true, @fork.forkable?)
      assert_equal(1, @fork_call)
      assert_equal(1, @waitpid_call)
      assert_equal(1732, @waitpid_pid)
    end

    def test_forkable_false
      @fork_not_implemented = true
      init_fork			# call Rucy::Fork#test_forkable?
      assert_equal(1, @fork_call)
      assert_equal(0, @waitpid_call)
      assert_equal(false, @fork.forkable?)
      assert_equal(1, @fork_call)
      assert_equal(0, @waitpid_call)
    end
  end
end
