# $Id: test_jconv.rb,v 1.3 2005/02/11 16:25:37 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'nkf'
require 'uconv'
require 'rucy/rep'
require 'rucy/jconv'

module TestRucy
  class TestJconv < RUNIT::TestCase
    def setup
      # dependency configuration for test
      @rep = Rucy::Repository.new

      # for NKF module
      @nkf_call = 0
      @nkf_option = nil
      @nkf_data = nil

      @NKF = Forwarder.new(self)
      class << @NKF
	def_delegator :__getobj__, :nkf
      end
      @rep.set_value(:NKF_module, @NKF)

      # for Uconv module
      @euctou8_call = 0
      @euctou8_data = nil
      @u8toeuc_call = 0
      @u8toeuc_data = nil
      @u8tou16_call = 0
      @u8tou16_data = nil

      @Uconv = Forwarder.new(self)
      class << @Uconv
	def_delegator :__getobj__, :euctou8
	def_delegator :__getobj__, :u8toeuc
	def_delegator :__getobj__, :u8tou16
      end
      @rep.set_value(:Uconv_module, @Uconv)

      # target
      @jconv = Rucy::Jconv.new(@rep)
    end

    # for NKF module

    def nkf(option, data)
      @nkf_call += 1
      @nkf_option = option
      @nkf_data = data
      NKF.nkf(option, data)
    end

    # for Uconv module

    def euctou8(data)
      @euctou8_call += 1
      @euctou8_data = data
      Uconv.euctou8(data)
    end

    def u8toeuc(data)
      @u8toeuc_call += 1
      @u8toeuc_data = data
      Uconv.u8toeuc(data)
    end

    def u8tou16(data)
      @u8tou16_call += 1
      @u8tou16_data = data
      Uconv.u8tou16(data)
    end

    # test

    def test_empty_string
      assert_equal('', @jconv.jtou8(''))
      assert_equal('', @jconv.u8toeuc(''))
      assert_equal(0, @nkf_call)
      assert_equal(0, @euctou8_call)
      assert_equal(0, @u8toeuc_call)
      assert_equal(0, @u8tou16_call)
    end

    def test_ascii
      assert_equal('Hello world.', @jconv.jtou8('Hello world.'))
      assert_equal(1, @u8tou16_call)
      assert_equal('Hello world.', @u8tou16_data)
      assert_equal(0, @nkf_call)
      assert_equal(0, @euctou8_call)

      assert_equal('Hello world.', @jconv.u8toeuc('Hello world.'))
      assert_equal(1, @u8toeuc_call)
      assert_equal('Hello world.', @u8toeuc_data)
    end

    HALO_UTF8 = "\343\201\223\343\202\223\343\201\253\343\201\241\343\201\257\343\200\202"
    HALO_EUC = "\244\263\244\363\244\313\244\301\244\317\241\243"
    HALO_SJIS = "\202\261\202\361\202\311\202\277\202\315\201B"

    def test_utf8
      assert_equal(HALO_UTF8, @jconv.jtou8(HALO_UTF8))
      assert_equal(1, @u8tou16_call)
      assert_equal(HALO_UTF8, @u8tou16_data)
      assert_equal(0, @nkf_call)
      assert_equal(0, @euctou8_call)
    end

    def test_ja_euc
      assert_equal(HALO_UTF8, @jconv.jtou8(HALO_EUC))
      assert_equal(1, @u8tou16_call)
      assert_equal(HALO_EUC, @u8tou16_data)
      assert_equal(1, @nkf_call)
      assert_equal('-e', @nkf_option)
      assert_equal(HALO_EUC, @nkf_data)
      assert_equal(1, @euctou8_call)
      assert_equal(HALO_EUC, @euctou8_data)

      assert_equal(HALO_EUC, @jconv.u8toeuc(HALO_UTF8))
      assert_equal(1, @u8toeuc_call)
      assert_equal(HALO_UTF8, @u8toeuc_data)
    end

    def test_ja_sjis
      assert_equal(HALO_UTF8, @jconv.jtou8(HALO_SJIS))
      assert_equal(1, @u8tou16_call)
      assert_equal(HALO_SJIS, @u8tou16_data)
      assert_equal(1, @nkf_call)
      assert_equal('-e', @nkf_option)
      assert_equal(HALO_SJIS, @nkf_data)
      assert_equal(1, @euctou8_call)
      assert_equal(HALO_EUC, @euctou8_data)
    end
  end
end
