#!/usr/local/bin/ruby
# $Id: test_params.rb,v 1.71 2005/03/06 13:31:47 toki Exp $

require 'rubyunit'
require 'pseudo_props'
require 'forwarder'
require 'rucy/rep'
require 'rucy/config'
require 'rucy/params'
require 'rucy/properties'

module TestRucy
  class TestAdminParams < RUNIT::TestCase
    include PseudoProperties

    def setup
      super

      # dependency configuration
      @rep = Rucy::Repository.new
      Rucy::Config.config(@rep)

      # test target
      @admin_params = Rucy::AdminParams.new(@rep)
    end

    def test_modified_count
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(0, @admin_params.modified_count)
      @admin_params.modified_count = 1
      assert_equal(1, @admin_params.modified_count)
      assert_equal(0, @modified_count)
      @admin_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert_equal(1, @modified_count)
    end

    def test_load_default_settings
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(%w[ admin_user admin_password localhost_only japanese_handling ].sort,
		   @params_names.sort)
      assert_equal('admin', @admin_params.admin_user)
      assert_equal('', @admin_params.admin_password)
      assert_equal(true, @admin_params.localhost_only)
      assert_equal(false, @admin_params.japanese_handling)
    end

    def test_load_admin_user
      @store['admin_user'] = 'alice'
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'admin_user'))
      assert_equal('alice', @admin_params.admin_user)
    end

    def test_load_admin_password
      @store['admin_password'] = 'himitsu'
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'admin_password'))
      assert_equal('himitsu', @admin_params.admin_password)
    end

    def test_load_localhost_only
      @store['localhost_only'] = false
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'localhost_only'))
      assert_equal(false, @admin_params.localhost_only)
    end

    def test_load_japanese_handling
      @store['japanese_handling'] = true
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'japanese_handling'))
      assert_equal(true, @admin_params.japanese_handling)
    end

    def test_save_admin_user
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('admin', @admin_params.admin_user)
      @admin_params.admin_user = 'alice'
      assert_equal('alice', @admin_params.admin_user)
      @admin_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert((@set_params_names.include? 'admin_user'))
      assert_equal('alice', @store['admin_user'])
    end

    def test_save_admin_password
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('', @admin_params.admin_password)
      @admin_params.admin_password = 'himitsu'
      assert_equal('himitsu', @admin_params.admin_password)
      @admin_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert((@set_params_names.include? 'admin_password'))
      assert_equal('himitsu', @store['admin_password'])
    end

    def test_save_localhost_only
      @store['admin_password'] = 'himitsu'
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert(! @admin_params.admin_password.empty?)
      assert_equal(true, @admin_params.localhost_only)
      @admin_params.localhost_only = false
      assert_equal(false, @admin_params.localhost_only)
      @admin_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert((@set_params_names.include? 'localhost_only'))
      assert_equal(false, @store['localhost_only'])
    end

    def test_save_japanese_handling
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(false, @admin_params.japanese_handling)
      @admin_params.japanese_handling = true
      assert_equal(true, @admin_params.japanese_handling)
      @admin_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert((@set_params_names.include? 'japanese_handling'))
      assert_equal(true, @store['japanese_handling'])
    end

    def test_null_admin_user
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('admin', @admin_params.admin_user)
      @admin_params.admin_user = nil
      assert_equal('admin', @admin_params.admin_user)
      @admin_params.admin_user = ' '
      assert_equal('admin', @admin_params.admin_user)
    end

    def test_admin_user_with_white_space
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('admin', @admin_params.admin_user)
      @admin_params.admin_user = ' alice '
      assert_equal('alice', @admin_params.admin_user)
    end

    def test_null_admin_password
      @store['admin_password'] = 'himitsu'
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('himitsu', @admin_params.admin_password)
      @admin_params.admin_password = nil
      assert_equal('', @admin_params.admin_password)
    end

    def test_no_effect_network_access
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('', @admin_params.admin_password)
      assert_equal(true, @admin_params.localhost_only)
      @admin_params.localhost_only = false
      assert_equal(true, @admin_params.localhost_only)
    end

    def test_no_effect_network_access_save_check
      @store['admin_password'] = ''
      @store['localhost_only'] = false
      @admin_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('', @admin_params.admin_password)
      assert_equal(false, @admin_params.localhost_only)
      @admin_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert((@set_params_names.include? 'admin_password'))
      assert((@set_params_names.include? 'localhost_only'))
      assert_equal('', @store['admin_password'])
      assert_equal(true, @store['localhost_only'])
    end
  end

  class TestServerParams < RUNIT::TestCase
    include PseudoProperties

    def setup
      super

      # dependency configuration
      @rep = Rucy::Repository.new
      Rucy::Config.config(@rep)

      # for BasicSocket class
      @BasicSocket = Forwarder.new(self)
      @rep.set_value(:BasicSocket_class, @BasicSocket)

      # for Rucy::Fork class
      @forkable_call = 0
      @forkable = true

      @fork = Forwarder.new(self)
      class << @fork
	def_delegator :__getobj__, :forkable?
      end

      # test target
      @server_params = Rucy::ServerParams.new(@fork, @rep)
    end

    # for Rucy::Fork class

    def forkable?
      @forkable_call += 1
      @forkable
    end

    # test

    def test_modified_count
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(0, @server_params.modified_count)
      @server_params.modified_count = 1
      assert_equal(1, @server_params.modified_count)
      assert_equal(0, @modified_count)
      @server_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert_equal(1, @modified_count)
    end

    def test_load_default_settings
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal([ 'server_type',
		     'port',
		     'bind_address',
		     'timeout',
		     'keep_alive',
		     'max_requests',
		     'queue_length',
		     'messengers',
		     'messenger_threads',
		     'messenger_queue_length',
		     'subprocess_user',
		     'subprocess_group',
		     'do_not_reverse_lookup'
                   ].sort,
		   @params_names.sort)
      assert_equal('multithread', @server_params.server_type)
      assert_equal(8888, @server_params.port)
      assert_equal('0.0.0.0', @server_params.bind_address)
      assert_equal(300, @server_params.timeout)
      assert_equal(8, @server_params.keep_alive)
      assert_equal(32, @server_params.max_requests)
      assert_equal(16, @server_params.queue_length)
      assert_equal(8, @server_params.messengers)
      assert_equal(4, @server_params.messenger_threads)
      assert_equal(4, @server_params.messenger_queue_length)
      assert_equal('nobody', @server_params.subprocess_user)
      assert_equal('nobody', @server_params.subprocess_group)
      assert_equal(false, @server_params.do_not_reverse_lookup)
    end

    def test_load_server_type_multithread
      @store['server_type'] = 'multithread'
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'server_type'))
      assert_equal(0, @forkable_call)
      assert_equal('multithread', @server_params.server_type)
    end

    def test_load_server_type_multiprocess
      @store['server_type'] = 'multiprocess'
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'server_type'))
      assert_equal(1, @forkable_call)
      assert_equal('multiprocess', @server_params.server_type)
    end

    def test_load_server_type_multiprocess_disabled
      @forkable = false
      @store['server_type'] = 'multiprocess'
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'server_type'))
      assert_equal(1, @forkable_call)
      assert_equal('multithread', @server_params.server_type)
    end

    def test_load_port
      @store['port'] = 80
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'port'))
      assert_equal(80, @server_params.port)
    end

    def test_load_bind_address
      @store['bind_address'] = 'localhost'
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'bind_address'))
      assert_equal('localhost', @server_params.bind_address)
    end

    def test_load_timeout
      @store['timeout'] = 60
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'timeout'))
      assert_equal(60, @server_params.timeout)
    end

    def test_load_keep_alive
      @store['keep_alive'] = 10
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'keep_alive'))
      assert_equal(10, @server_params.keep_alive)
    end

    def test_load_max_requests
      @store['max_requests'] = 256
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'max_requests'))
      assert_equal(256, @server_params.max_requests)
    end

    def test_load_queue_length
      @store['queue_length'] = 64
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'queue_length'))
      assert_equal(64, @server_params.queue_length)
    end

    def test_load_messengers
      @store['messengers'] = 128
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'messengers'))
      assert_equal(128, @server_params.messengers)
    end

    def test_load_messenger_threads
      @store['messenger_threads'] = 512
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'messenger_threads'))
      assert_equal(512, @server_params.messenger_threads)
    end

    def test_load_messenger_queue_length
      @store['messenger_queue_length'] = 256
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'messenger_queue_length'))
      assert_equal(256, @server_params.messenger_queue_length)
    end

    def test_load_subprocess_user
      @store['subprocess_user'] = 'httpd'
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'subprocess_user'))
      assert_equal('httpd', @server_params.subprocess_user)
    end

    def test_load_subprocess_group
      @store['subprocess_group'] = 'httpd'
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'subprocess_group'))
      assert_equal('httpd', @server_params.subprocess_group)
    end

    def test_load_do_not_reverse_lookup
      @store['do_not_reverse_lookup'] = true
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'do_not_reverse_lookup'))
      assert_equal(true, @server_params.do_not_reverse_lookup)
    end

    def test_save_server_type_multithread
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('multithread', @server_params.server_type)
      @server_params.server_type = 'multithread'
      assert_equal(0, @forkable_call)
      assert_equal('multithread', @server_params.server_type)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'server_type'))
      assert_equal('multithread', @store['server_type'])
    end

    def test_save_server_type_multiprocess
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('multithread', @server_params.server_type)
      @server_params.server_type = 'multiprocess'
      assert_equal(1, @forkable_call)
      assert_equal('multiprocess', @server_params.server_type)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'server_type'))
      assert_equal('multiprocess', @store['server_type'])
    end

    def test_save_server_type_multiprocess_disabled
      @forkable = false
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('multithread', @server_params.server_type)
      assert_exception(RuntimeError) {
	@server_params.server_type = 'multiprocess'
      }
      assert_equal(1, @forkable_call)
      assert_equal('multithread', @server_params.server_type)
    end

    def test_save_port
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(8888, @server_params.port)
      @server_params.port = 80
      assert_equal(80, @server_params.port)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'port'))
      assert_equal(80, @store['port'])
    end

    def test_save_bind_address
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('0.0.0.0', @server_params.bind_address)
      @server_params.bind_address = 'localhost'
      assert_equal('localhost', @server_params.bind_address)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'bind_address'))
      assert_equal('localhost', @store['bind_address'])
    end

    def test_save_timeout
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(300, @server_params.timeout)
      @server_params.timeout = 60
      assert_equal(60, @server_params.timeout)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'timeout'))
      assert_equal(60, @store['timeout'])
    end

    def test_save_keep_alive
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(8, @server_params.keep_alive)
      @server_params.keep_alive = 30
      assert_equal(30, @server_params.keep_alive)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'keep_alive'))
      assert_equal(30, @store['keep_alive'])
    end

    def test_save_max_requests
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(32, @server_params.max_requests)
      @server_params.max_requests = 256
      assert_equal(256, @server_params.max_requests)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'max_requests'))
      assert_equal(256, @store['max_requests'])
    end

    def test_save_queue_length
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(16, @server_params.queue_length)
      @server_params.queue_length = 64
      assert_equal(64, @server_params.queue_length)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'queue_length'))
      assert_equal(64, @store['queue_length'])
    end

    def test_save_messengers
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(8, @server_params.messengers)
      @server_params.messengers = 16
      assert_equal(16, @server_params.messengers)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'messengers'))
      assert_equal(16, @store['messengers'])
    end

    def test_save_messenger_threads
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(4, @server_params.messenger_threads)
      @server_params.messenger_threads = 32
      assert_equal(32, @server_params.messenger_threads)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'messenger_threads'))
      assert_equal(32, @store['messenger_threads'])
    end

    def test_save_messenger_queue_length
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(4, @server_params.messenger_queue_length)
      @server_params.messenger_queue_length = 16
      assert_equal(16, @server_params.messenger_queue_length)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'messenger_queue_length'))
      assert_equal(16, @store['messenger_queue_length'])
    end

    def test_save_subprocess_user
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('nobody', @server_params.subprocess_user)
      @server_params.subprocess_user = 'httpd'
      assert_equal('httpd', @server_params.subprocess_user)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'subprocess_user'))
      assert_equal('httpd', @store['subprocess_user'])
    end

    def test_save_subprocess_group
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('nobody', @server_params.subprocess_group)
      @server_params.subprocess_group = 'httpd'
      assert_equal('httpd', @server_params.subprocess_group)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'subprocess_group'))
      assert_equal('httpd', @store['subprocess_group'])
    end

    def test_save_do_not_reverse_lookup
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(false, @server_params.do_not_reverse_lookup)
      @server_params.do_not_reverse_lookup = true
      assert_equal(true, @server_params.do_not_reverse_lookup)
      @server_params.save(@properties)
      assert((@set_params_names.include? 'do_not_reverse_lookup'))
      assert_equal(true, @store['do_not_reverse_lookup'])
    end

    def test_invalid_server_type
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('multithread', @server_params.server_type)
      assert_exception(RuntimeError) {
	@server_params.server_type = 'foo'
      }
      assert_equal('multithread', @server_params.server_type)
    end

    def test_invalid_port_number
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(8888, @server_params.port)
      assert_exception(RuntimeError) {
	@server_params.port = -1
      }
      assert_equal(8888, @server_params.port)
    end

    def test_empty_bind_address
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('0.0.0.0', @server_params.bind_address)
      @server_params.bind_address = ''
      assert_equal(nil, @server_params.bind_address)
      @server_params.bind_address = ' '
      assert_equal(nil, @server_params.bind_address)
    end

    def test_invalid_timeout
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(300, @server_params.timeout)
      assert_exception(RuntimeError) {
	@server_params.timeout = 0
      }
      assert_exception(RuntimeError) {
	@server_params.timeout = -1
      }
      assert_equal(300, @server_params.timeout)
    end

    def test_invalid_keep_alive
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(8, @server_params.keep_alive)
      assert_exception(RuntimeError) {
	@server_params.keep_alive = -1
      }
      assert_equal(8, @server_params.keep_alive)
    end

    def test_invalid_max_requests
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(32, @server_params.max_requests)
      assert_exception(RuntimeError) {
	@server_params.max_requests = 0
      }
      assert_exception(RuntimeError) {
	@server_params.max_requests = -1
      }
      assert_equal(32, @server_params.max_requests)
    end

    def test_invalid_queue_length
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(16, @server_params.queue_length)
      assert_exception(RuntimeError) {
	@server_params.queue_length = 0
      }
      assert_exception(RuntimeError) {
	@server_params.queue_length = -1
      }
      assert_equal(16, @server_params.queue_length)
    end

    def test_invalid_messengers
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(8, @server_params.messengers)
      assert_exception(RuntimeError) {
	@server_params.messengers = 0
      }
      assert_exception(RuntimeError) {
	@server_params.messengers = -1
      }
      assert_equal(8, @server_params.messengers)
    end

    def test_invalid_messenger_threads
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(4, @server_params.messenger_threads)
      assert_exception(RuntimeError) {
	@server_params.messenger_threads = 0
      }
      assert_exception(RuntimeError) {
	@server_params.messenger_threads = -1
      }
      assert_equal(4, @server_params.messenger_threads)
    end

    def test_invalid_messenger_queue_length
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(4, @server_params.messenger_queue_length)
      assert_exception(RuntimeError) {
	@server_params.messenger_queue_length = 0
      }
      assert_exception(RuntimeError) {
	@server_params.messenger_queue_length = -1
      }
      assert_equal(4, @server_params.messenger_queue_length)
    end

    def test_invalid_subprocess_user
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('nobody', @server_params.subprocess_user)
      assert_exception(RuntimeError) {
	@server_params.subprocess_user = -1
      }
      assert_exception(RuntimeError) {
	@server_params.subprocess_user = :nobody
      }
      assert_equal('nobody', @server_params.subprocess_user)
    end

    def test_invalid_subprocess_group
      @server_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('nobody', @server_params.subprocess_group)
      assert_exception(RuntimeError) {
	@server_params.subprocess_group = -1
      }
      assert_exception(RuntimeError) {
	@server_params.subprocess_group = :nobody
      }
      assert_equal('nobody', @server_params.subprocess_group)
    end

    def test_multiprocess_disabled_false
      @forkable = true
      assert_equal(false, @server_params.multiprocess_disabled?)
      assert_equal(1, @forkable_call)
    end

    def test_multiprocess_disabled_true
      @forkable = false
      assert_equal(true, @server_params.multiprocess_disabled?)
      assert_equal(1, @forkable_call)
    end
  end

  class TestLoggingParams < RUNIT::TestCase
    include PseudoProperties

    def setup
      super

      # dependency configuration
      @rep = Rucy::Repository.new
      Rucy::Config.config(@rep)

      # test target
      @logging_params = Rucy::LoggingParams.new(@rep)
    end

    def test_modified_count
      @logging_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(0, @logging_params.modified_count)
      @logging_params.modified_count = 1
      assert_equal(1, @logging_params.modified_count)
      assert_equal(0, @modified_count)
      @logging_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert_equal(1, @modified_count)
    end

    def test_load_default_settings
      @logging_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(%w[ stdout_logging_level logfiles ].sort, @params_names.sort)
      assert_equal('notice', @logging_params.stdout_logging_level)
      assert([], @logging_params.logfiles)
    end

    def test_load_stdout_logging_level
      @store['stdout_logging_level'] = 'debug'
      @logging_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'stdout_logging_level'))
      assert_equal('debug', @logging_params.stdout_logging_level)
    end

    def test_save_stdout_logging_level
      @logging_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal('notice', @logging_params.stdout_logging_level)
      @logging_params.stdout_logging_level = 'debug'
      assert_equal('debug', @logging_params.stdout_logging_level)
      @logging_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert((@set_params_names.include? 'stdout_logging_level'))
      assert_equal('debug', @store['stdout_logging_level'])
    end

    def test_load_logfiles
      @store['logfiles'] = [
	{ 'path' => 'rucy.log',
	  'logging_level' => 'info',
	}
      ]
      @logging_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'logfiles'))
      assert_equal(1, @logging_params.logfiles.length)
      assert_equal('rucy.log', @logging_params.logfiles[0].path)
      assert_equal('info', @logging_params.logfiles[0].logging_level)
    end

    def test_save_logfiles
      @logging_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(0, @logging_params.logfiles.length)
      @logging_params.add_logfile
      assert_equal(1, @logging_params.logfiles.length)
      assert_equal(nil, @logging_params.logfiles[0].path)
      assert_equal('notice', @logging_params.logfiles[0].logging_level)

      @logging_params.logfiles[0].path = 'debug.log'
      assert_equal('debug.log', @logging_params.logfiles[0].path)
      @logging_params.logfiles[0].logging_level = 'debug'
      assert_equal('debug', @logging_params.logfiles[0].logging_level)

      @logging_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert((@set_params_names.include? 'logfiles'))
      assert_equal(1, @store['logfiles'].length)
      assert_equal([ { 'path' => 'debug.log',
		       'logging_level' => 'debug'
		     }
		   ], @store['logfiles'])
    end

    def test_delete_logfile_at
      @store['logfiles'] = [
	{ 'path' => 'rucy.log',
	  'logging_level' => 'info',
	}
      ]
      @logging_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(1, @logging_params.logfiles.length)
      @logging_params.del_logfile_at(0)
      assert_equal(0, @logging_params.logfiles.length)
      @logging_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert_equal([], @store['logfiles'])
    end

    def test_invalid_logfiles
      @logging_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(0, @logging_params.logfiles.length)
      @logging_params.add_logfile

      @logging_params.logfiles[0].path = nil
      assert_equal(nil, @logging_params.logfiles[0].path)
      assert_exception(RuntimeError) {
	@logging_params.save(@properties)
      }

      @logging_params.logfiles[0].path = ' '
      assert_equal(' ', @logging_params.logfiles[0].path)
      assert_exception(RuntimeError) {
	@logging_params.save(@properties)
      }

      assert_exception(RuntimeError) {
	@logging_params.logfiles[0].logging_level = 'detarame'
      }
    end
  end

  class TestAccessLogParams < RUNIT::TestCase
    include PseudoProperties

    def setup
      super

      # dependency configuration
      @rep = Rucy::Repository.new
      Rucy::Config.config(@rep)

      # target
      @access_log_params = Rucy::AccessLogParams.new(@rep)
    end

    def test_modified_count
      @access_log_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(0, @access_log_params.modified_count)
      @access_log_params.modified_count = 1
      assert_equal(1, @access_log_params.modified_count)
      assert_equal(0, @modified_count)
      @access_log_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert_equal(1, @modified_count)
    end

    def test_load_default_settings
      @access_log_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal([ 'access_log_logging',
		     'access_log_format',
		     'access_logs'
		   ].sort, @params_names.sort)
      assert_equal(true, @access_log_params.logging)
      assert_equal(Rucy::AccessLog::COMMON_LOG_FORMAT, @access_log_params.format)
      assert_equal(0, @access_log_params.logfiles.length)
    end

    def test_load_logging
      @store['access_log_logging'] = false
      @access_log_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'access_log_logging'))
      assert_equal(false, @access_log_params.logging)
    end

    def test_load_format
      @store['access_log_format'] = 'HALO'
      @access_log_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'access_log_format'))
      assert_equal('HALO', @access_log_params.format)
    end

    def test_load_logfiles
      @store['access_logs'] = [ { 'path' => 'access.log', 'format' => 'HALO' } ]
      @access_log_params.load(@properties)
      assert_equal(1, @params_call)
      assert((@params_names.include? 'access_logs'))
      assert_equal(1, @access_log_params.logfiles.length)
      assert_equal('access.log', @access_log_params.logfiles[0].path)
      assert_equal('HALO', @access_log_params.logfiles[0].format)
    end

    def test_save_logging
      @access_log_params.load(@properties)
      assert_equal(true, @access_log_params.logging)
      @access_log_params.logging = false
      assert_equal(false, @access_log_params.logging)
      @access_log_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert((@set_params_names.include? 'access_log_logging'))
      assert_equal(false, @store['access_log_logging'])
    end

    def test_save_format
      @store['access_log_format'] = 'foo'
      @access_log_params.load(@properties)
      assert_equal('foo', @access_log_params.format)
      @access_log_params.format = 'bar'
      assert_equal('bar', @access_log_params.format)
      @access_log_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert((@set_params_names.include? 'access_log_format'))
      assert_equal('bar', @store['access_log_format'])
    end

    def test_save_logfiles
      @store['access_logs'] = [ { 'path' => nil, 'format' => nil } ]
      @access_log_params.load(@properties)
      assert_equal(1, @access_log_params.logfiles.length)
      assert_equal(nil, @access_log_params.logfiles[0].path)
      assert_equal(nil, @access_log_params.logfiles[0].format)
      @access_log_params.logfiles[0].path = 'access.log'
      @access_log_params.logfiles[0].format = 'HALO'
      assert_equal('access.log', @access_log_params.logfiles[0].path)
      assert_equal('HALO', @access_log_params.logfiles[0].format)
      @access_log_params.save(@properties)
      assert_equal(1, @set_params_call)
      assert((@set_params_names.include? 'access_logs'))
      assert_equal([ { 'path' => 'access.log', 'format' => 'HALO' } ], @store['access_logs'])
    end

    def test_add_logfile
      @access_log_params.load(@properties)
      assert_equal(0, @access_log_params.logfiles.length)
      @access_log_params.add_logfile
      assert_equal(1, @access_log_params.logfiles.length)
      assert_equal(nil, @access_log_params.logfiles[0].path)
      assert_equal(Rucy::AccessLog::COMMON_LOG_FORMAT, @access_log_params.logfiles[0].format)
    end

    def test_del_logfile_at
      @store['access_logs'] = [ { 'path' => nil, 'format' => nil } ]
      @access_log_params.load(@properties)
      assert_equal(1, @access_log_params.logfiles.length)
      @access_log_params.del_logfile_at(0)
      assert_equal(0, @access_log_params.logfiles.length)
    end

    def test_invalid_logfiles
      @access_log_params.load(@properties)
      assert_equal(1, @params_call)
      assert_equal(0, @access_log_params.logfiles.length)

      @access_log_params.format = nil
      assert_exception(RuntimeError) {
	@access_log_params.save(@properties)
      }
      @access_log_params.format = ''
      assert_exception(RuntimeError) {
	@access_log_params.save(@properties)
      }
      @access_log_params.format = ' '
      assert_exception(RuntimeError) {
	@access_log_params.save(@properties)
      }

      @access_log_params.format = 'test'
      @access_log_params.add_logfile

      @access_log_params.logfiles[0].format = 'foo'
      @access_log_params.logfiles[0].path = nil
      assert_exception(RuntimeError) {
	@access_log_params.save(@properties)
      }
      @access_log_params.logfiles[0].path = ''
      assert_exception(RuntimeError) {
	@access_log_params.save(@properties)
      }
      @access_log_params.logfiles[0].path = ' '
      assert_exception(RuntimeError) {
	@access_log_params.save(@properties)
      }

      @access_log_params.logfiles[0].path = 'foo.log'
      @access_log_params.logfiles[0].format = nil
      assert_exception(RuntimeError) {
	@access_log_params.save(@properties)
      }
      @access_log_params.logfiles[0].format = ''
      assert_exception(RuntimeError) {
	@access_log_params.save(@properties)
      }
      @access_log_params.logfiles[0].format = ' '
      assert_exception(RuntimeError) {
	@access_log_params.save(@properties)
      }
    end
  end

  class TestAliasList < RUNIT::TestCase
    include PseudoProperties

    def setup
      super

      # dependency configuration
      @rep = Rucy::Repository.new
      Rucy::Config.config(@rep)

      # test target
      @alias_list = Rucy::AliasList.new(@rep)
    end

    def test_modified_count
      @alias_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal(0, @alias_list.modified_count)
      @alias_list.modified_count = 1
      assert_equal(1, @alias_list.modified_count)
      assert_equal(0, @modified_count)
      @alias_list.save(@properties)
      assert_equal(1, @set_list_call)
      assert_equal(1, @modified_count)
    end

    def test_load_default_settings
      @alias_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal('aliases', @list_name)
      assert(@alias_list.empty?)
      assert_equal(0, @alias_list.size)
      assert_equal(0, @alias_list.length)
      for alias_params in @alias_list
	assert_fail('found an element in empty list.')
      end
    end

    def test_load
      @store['aliases'] = [
	{ 'alias_path' => '/',
	  'orig_path' => '/index.rb',
	  'virtual_host' => 'foo'
	}
      ]
      @alias_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal('aliases', @list_name)
      assert(! @alias_list.empty?)
      assert_equal(1, @alias_list.size)
      assert_equal(1, @alias_list.length)

      # test of operator `[]'
      assert_equal('/', @alias_list[0].alias_path)
      assert_equal('/index.rb', @alias_list[0].orig_path)
      assert_equal('foo', @alias_list[0].virtual_host)

      # test of each
      count = 0
      for params in @alias_list
	count += 1
	assert_equal('/', params.alias_path)
	assert_equal('/index.rb', params.orig_path)
	assert_equal('foo', params.virtual_host)
      end
      assert_equal(1, count)
    end

    def test_save
      @store['aliases'] = [
	{ 'alias_path' => nil,
	  'orig_path' => nil,
	  'virtual_host' => nil
	}
      ]
      @alias_list.load(@properties)
      @alias_list[0].alias_path = '/'
      @alias_list[0].orig_path = '/index.rb'
      @alias_list[0].virtual_host = 'foo'
      @alias_list.save(@properties)
      assert_equal(1, @set_list_call)
      assert_equal('aliases', @set_list_name)
      assert_equal([ { 'alias_path' => '/',
		       'orig_path' => '/index.rb',
		       'virtual_host' => 'foo'
		     }
		   ], @store['aliases'])
    end

    def test_add_entry
      @alias_list.load(@properties)
      assert(@alias_list.empty?)
      assert_equal(0, @alias_list.size)
      assert_equal(0, @alias_list.length)
      @alias_list.add_entry
      assert(! @alias_list.empty?)
      assert_equal(1, @alias_list.size)
      assert_equal(1, @alias_list.length)
      assert_equal(nil, @alias_list[0].alias_path)
      assert_equal(nil, @alias_list[0].orig_path)
      assert_equal(nil, @alias_list[0].virtual_host)
    end

    def test_delete_at
      @store['aliases'] = [
	{ 'alias_path' => '/foo',
	  'orig_path' => '/foo.rb',
	  'virtual_host' => nil
	}
      ]
      @alias_list.load(@properties)
      assert(! @alias_list.empty?)
      assert_equal(1, @alias_list.size)
      assert_equal(1, @alias_list.length)
      assert_equal('/foo', @alias_list[0].alias_path)
      assert_equal('/foo.rb', @alias_list[0].orig_path)
      assert_equal(nil, @alias_list[0].virtual_host)
      @alias_list.delete_at(0)
      assert(@alias_list.empty?)
      assert_equal(0, @alias_list.size)
      assert_equal(0, @alias_list.length)
    end

    def test_swap
      @store['aliases'] = [
	{ 'alias_path' => '/foo',
	  'orig_path' => '/foo.rb',
	  'virtual_host' => nil
	},
	{ 'alias_path' => '/bar',
	  'orig_path' => '/bar.rb',
	  'virtual_host' => 'bar'
	}
      ]
      @alias_list.load(@properties)
      assert_equal('/foo', @alias_list[0].alias_path)
      assert_equal('/foo.rb', @alias_list[0].orig_path)
      assert_equal(nil, @alias_list[0].virtual_host)
      assert_equal('/bar', @alias_list[1].alias_path)
      assert_equal('/bar.rb', @alias_list[1].orig_path)
      assert_equal('bar', @alias_list[1].virtual_host)
      @alias_list.swap(0, 1)
      assert_equal('/bar', @alias_list[0].alias_path)
      assert_equal('/bar.rb', @alias_list[0].orig_path)
      assert_equal('bar', @alias_list[0].virtual_host)
      assert_equal('/foo', @alias_list[1].alias_path)
      assert_equal('/foo.rb', @alias_list[1].orig_path)
      assert_equal(nil, @alias_list[1].virtual_host)
    end

    def test_inavlid_alias_path
      @store['aliases'] = [
	{ 'alias_path' => '/foo',
	  'orig_path' => '/foo.rb',
	  'virtual_host' => nil
	}
      ]
      @alias_list.load(@properties)
      assert_equal('/foo', @alias_list[0].alias_path)
      assert_equal('/foo.rb', @alias_list[0].orig_path)
      assert_equal(nil, @alias_list[0].virtual_host)

      @alias_list[0].alias_path = nil
      assert_exception(RuntimeError) {
	@alias_list.save(@properties)
      }

      @alias_list[0].alias_path = ''
      assert_exception(RuntimeError) {
	@alias_list.save(@properties)
      }

      @alias_list[0].alias_path = ' '
      assert_exception(RuntimeError) {
	@alias_list.save(@properties)
      }

      assert_exception(RuntimeError) {
	@alias_list[0].alias_path = 123
      }
      assert_exception(RuntimeError) {
	@alias_list[0].alias_path = false
      }
    end

    def test_inavlid_orig_path
      @store['aliases'] = [
	{ 'alias_path' => '/foo',
	  'orig_path' => '/foo.rb',
	  'virtual_host' => nil
	}
      ]
      @alias_list.load(@properties)
      assert_equal('/foo', @alias_list[0].alias_path)
      assert_equal('/foo.rb', @alias_list[0].orig_path)
      assert_equal(nil, @alias_list[0].virtual_host)

      @alias_list[0].orig_path = nil
      assert_exception(RuntimeError) {
	@alias_list.save(@properties)
      }

      @alias_list[0].orig_path = ''
      assert_exception(RuntimeError) {
	@alias_list.save(@properties)
      }

      @alias_list[0].orig_path = ' '
      assert_exception(RuntimeError) {
	@alias_list.save(@properties)
      }

      assert_exception(RuntimeError) {
	@alias_list[0].orig_path = 123
      }
      assert_exception(RuntimeError) {
	@alias_list[0].orig_path = false
      }
    end

    def test_invalid_virtual_host
      @store['aliases'] = [
	{ 'alias_path' => '/foo',
	  'orig_path' => '/foo.rb',
	  'virtual_host' => nil
	}
      ]
      @alias_list.load(@properties)
      assert_equal('/foo', @alias_list[0].alias_path)
      assert_equal('/foo.rb', @alias_list[0].orig_path)
      assert_equal(nil, @alias_list[0].virtual_host)

      @alias_list[0].virtual_host = nil	# allowed
      assert_exception(RuntimeError) {
	@alias_list[0].virtual_host = 123
      }
      assert_exception(RuntimeError) {
	@alias_list[0].virtual_host = false
      }
    end

    def test_empty_virtual_host
      @store['aliases'] = [
	{ 'alias_path' => nil,
	  'orig_path' => nil,
	  'virtual_host' => nil
	}
      ]
      @alias_list.load(@properties)
      assert_equal(nil, @alias_list[0].virtual_host)
      @alias_list[0].virtual_host = ''
      assert_equal(nil, @alias_list[0].virtual_host)
      @alias_list[0].virtual_host = ' '
      assert_equal(nil, @alias_list[0].virtual_host)
    end
  end

  class TestListEntry < RUNIT::TestCase
    class ListEntryStub < Rucy::ListEntry
      extend Forwardable

      def initialize(attrs, forwarder)
	super(attrs)
	@forwarder = forwarder
      end

      def_delegator :@forwarder, :name_tag
      def_delegator :@forwarder, :path_tag
      def_delegator :@forwarder, :mask_tag
      def_delegator :@forwarder, :name_list
      def_delegator :@forwarder, :arg_info_alist
    end

    def setup
      # implementation of abstract methods for Rucy::ListEntry class
      @name_tag_call = 0
      @path_tag_call = 0
      @mask_tag_call = 0
      @name_list_call = 0
      @arg_info_alist_call = 0
      @arg_info_alist_name = nil

      # target
      @attrs = {
	'document' => nil,
	'arguments' => {},
	'mount_path' => nil,
	'mount_mask' => nil,
	'virtual_host' => nil,
	'comment' => nil
      }
      @entry = ListEntryStub.new(@attrs, self)
    end

    # implementation of abstract methods for Rucy::ListEntry class

    def name_tag
      @name_tag_call += 1
      'document'
    end

    def path_tag
      @path_tag_call += 1
      'mount_path'
    end

    def mask_tag
      @mask_tag_call += 1
      'mount_mask'
    end

    def name_list
      @name_list_call += 1
      [ 'Foo' ]
    end

    def arg_info_alist(name)
      @arg_info_alist_call += 1
      @arg_info_alist_name = name
      [ [ 'str_arg',    'string',   :string,   'hi' ],
	[ 'passwd_arg', 'password', :password, nil ],
	[ 'txt_arg',    'text',     :text,     '<html>Hello world.</html>' ],
	[ 'num_arg',    'number',   :integer,  0 ],
	[ 'reg_arg',    'regexp',   :regexp,   /foo/ ],
	[ 'bool_arg',   'boolean',  :bool,     true ],
	[ 'sel_arg',    'select',   :select,   %w[ foo bar baz ] ],
	[ 'radio_arg',  'radio',    :radio,    %w[ foo bar baz ] ],
	[ 'chk_arg',    'checkset', :checkset,
	  [ [ 'foo', false ],
	    [ 'bar', false ],
	    [ 'baz', false ]
	  ]
	]
      ]
    end

    # test

    def test_tags
      assert_equal('document', @entry.name_tag)
      assert_equal(1, @name_tag_call)
      assert_equal('mount_path', @entry.path_tag)
      assert_equal(1, @path_tag_call)
      assert_equal('mount_mask', @entry.mask_tag)
      assert_equal(1, @mask_tag_call)
    end

    def test_name_list
      assert_equal([ 'Foo' ], @entry.name_list)
      assert_equal(1, @name_list_call)
    end

    def test_arg_info_alist
      assert_equal([ [ 'str_arg',    'string',   :string,   'hi' ],
		     [ 'passwd_arg', 'password', :password, nil ],
		     [ 'txt_arg',    'text',     :text,     '<html>Hello world.</html>' ],
		     [ 'num_arg',    'number',   :integer,  0 ],
		     [ 'reg_arg',    'regexp',   :regexp,   /foo/ ],
		     [ 'bool_arg',   'boolean',  :bool,     true ],
		     [ 'sel_arg',    'select',   :select,   %w[ foo bar baz ] ],
		     [ 'radio_arg',  'radio',    :radio,    %w[ foo bar baz ] ],
		     [ 'chk_arg',    'checkset', :checkset,
		       [ [ 'foo', false ],
			 [ 'bar', false ],
			 [ 'baz', false ]
		       ]
		     ]
		   ], @entry.arg_info_alist('Foo'))
      assert_equal(1, @arg_info_alist_call)
      assert_equal('Foo', @arg_info_alist_name)
    end

    def test_name
      @entry.name = 'Foo'
      assert_equal('Foo', @attrs['document'])
      assert_equal(1, @name_tag_call)
      assert_equal(1, @name_list_call)
      assert_equal('Foo', @entry.name)
      assert_equal(2, @name_tag_call)
      assert_exception(RuntimeError) {
	@entry.name = nil
      }
      assert_exception(RuntimeError) {
	@entry.name = 'Bar'
      }
    end

    def test_path
      @entry.path = '/foo'
      assert_equal('/foo', @attrs['mount_path'])
      assert_equal(1, @path_tag_call)
      assert_equal('/foo', @entry.path)
      assert_equal(2, @path_tag_call)
      #assert_exception(RuntimeError) {
	@entry.path = nil	# checked at saving
      #}
      #assert_exception(RuntimeError) {
	@entry.path = 'foo'	# checked at saving
      #}
      assert_exception(RuntimeError) {
	@entry.path = 0
      }
    end

    def test_mask
      @entry.mask = /foo/
      assert_equal(/foo/, @attrs['mount_mask'])
      assert_equal(1, @mask_tag_call)
      assert_equal(/foo/, @entry.mask)
      assert_equal(2, @mask_tag_call)
    end

    def test_virtual_host
      assert_equal(nil, @entry.virtual_host)
      @entry.virtual_host = 'www.foo.net'
      assert_equal('www.foo.net', @attrs['virtual_host'])
      assert_equal('www.foo.net', @entry.virtual_host)
      @entry.virtual_host = nil	# allowed
      assert_exception(RuntimeError) {
	@entry.virtual_host = 0	# not allowed
      }
    end

    def test_comment
      assert_equal(nil, @entry.comment)
      @entry.comment = 'HELO'
      assert_equal('HELO', @attrs['comment'])
      assert_equal('HELO', @entry.comment)
      @entry.comment = nil	# allowed
      assert_exception(RuntimeError) {
	@entry.comment = 0	# not allowed
      }
    end

    def test_init
      @entry.init('Foo')
      assert_equal(1, @name_tag_call)
      assert_equal(1, @name_list_call)
      assert_equal('Foo', @attrs['document'])
      assert_equal(1, @arg_info_alist_call)
      assert_equal('Foo', @arg_info_alist_name)
      assert_equal({ 'str_arg' =>  'hi',
		     'passwd_arg' => nil,
		     'txt_arg' => '<html>Hello world.</html>',
		     'num_arg' => 0,
		     'reg_arg' => /foo/,
		     'bool_arg' => true,
		     'sel_arg' => 'foo',
		     'radio_arg' => 'foo',
		     'chk_arg' => {
		       'foo'=> false,
		       'bar' => false,
		       'baz' => false
		     }
		   }, @attrs['arguments'])

    end

    def init_arguments
      @attrs['document'] = 'Foo'
      @attrs['arguments'] = {
	'str_arg'    =>  'hi',
	'passwd_arg' => nil,
	'txt_arg'    => '<html>Hello world.</html>',
	'num_arg'    => 0,
	'reg_arg'    => /foo/,
	'bool_arg'   => true,
	'sel_arg'    => 'foo',
	'radio_arg'  => 'foo',
	'chk_arg'    => {
	  'foo'=> false,
	  'bar' => false,
	  'baz' => false
	}
      }
    end

    def test_args
      init_arguments
      assert_equal([ 'hi',	# string
		     nil,	# password
		     '<html>Hello world.</html>', # text
		     0,		# number
		     /foo/,	# regexp
		     true,	# bool
		     'foo',	# select
		     'foo',	# radio
		     { 'foo'=> false, # checkset
		       'bar' => false,
		       'baz' => false
		     }
		   ], @entry.args)
      assert_equal(1, @arg_info_alist_call)
      assert_equal('Foo', @arg_info_alist_name)
      @entry.args.clear
      assert_equal([ 'hi',	# string
		     nil,	# password
		     '<html>Hello world.</html>', # text
		     0,		# number
		     /foo/,	# regexp
		     true,	# bool
		     'foo',	# select
		     'foo',	# radio
		     { 'foo'=> false, # checkset
		       'bar' => false,
		       'baz' => false
		     }
		   ], @entry.args)
    end

    def test_args2
      init_arguments
      expected_args2 = [
	{ :name => 'string',
	  :type => :string,
	  :default => 'hi',
	  :value => 'hi'
	},
	{ :name => 'password',
	  :type => :password,
	  :default => nil,
	  :value => nil
	},
	{ :name => 'text',
	  :type => :text,
	  :default => '<html>Hello world.</html>',
	  :value => '<html>Hello world.</html>'
	},
	{ :name => 'number',
	  :type => :integer,
	  :default => 0,
	  :value => 0
	},
	{ :name => 'regexp',
	  :type => :regexp,
	  :default => /foo/,
	  :value => /foo/
	},
	{ :name => 'boolean',
	  :type => :bool,
	  :default => true,
	  :value => true
	},
	{ :name => 'select',
	  :type => :select,
	  :default => %w[ foo bar baz ],
	  :value => 'foo'
	},
	{ :name => 'radio',
	  :type => :radio,
	  :default => %w[ foo bar baz ],
	  :value => 'foo'
	},
	{ :name => 'checkset',
	  :type => :checkset,
	  :default => [
	    [ 'foo', false ],
	    [ 'bar', false ],
	    [ 'baz', false ]
	  ],
	  :value => {
	    'foo' => false,
	    'bar' => false,
	    'baz' => false
	  }
	}
      ]
      assert_equal(expected_args2, @entry.args2)
      assert_equal(1, @name_tag_call)
      assert_equal(1, @arg_info_alist_call)
      assert_equal('Foo', @arg_info_alist_name)
      for arg_info in @entry.args2
	arg_info.clear
      end
      @entry.args2.clear
      assert_equal(expected_args2, @entry.args2)
    end

    def test_argument_string
      init_arguments
      assert_equal('hi', @entry.args[0])
      @entry.set_arg_at(0, 'hello')
      assert_equal('hello', @entry.args[0])
      @entry.set_arg_at(0, nil)	# allowed
      assert_exception(RuntimeError) {
	@entry.set_arg_at(0, :hi) # not allowed
      }
    end

    def test_argument_password
      init_arguments
      assert_equal(nil, @entry.args[1])
      @entry.set_arg_at(1, 'secret')
      assert_equal('secret', @entry.args[1])
      @entry.set_arg_at(1, nil)	# allowd
      assert_exception(RuntimeError) {
	@entry.set_arg_at(1, :secret) # not allowed
      }
    end

    def test_argument_text
      init_arguments
      assert_equal('<html>Hello world.</html>', @entry.args[2])
      @entry.set_arg_at(2, 'Hello world.')
      assert_equal('Hello world.', @entry.args[2])
      @entry.set_arg_at(2, nil)	# allowed
      assert_exception(RuntimeError) {
	@entry.set_arg_at(2, :Hello_world) # not allowed
      }
    end

    def test_argument_number
      init_arguments
      assert_equal(0, @entry.args[3])
      @entry.set_arg_at(3, 9999)
      assert_equal(9999, @entry.args[3])
      @entry.set_arg_at(3, nil)	# allowed
      assert_exception(RuntimeError) {
	@entry.set_arg_at(3, '9999') # not allowed
      }
    end

    def test_argument_regexp
      init_arguments
      assert_equal(/foo/, @entry.args[4])
      @entry.set_arg_at(4, /bar/)
      assert_equal(/bar/, @entry.args[4])
      @entry.set_arg_at(4, nil)	# allowed
      assert_exception(RuntimeError) {
	@entry.set_arg_at(4, 'foo') # not allowed
      }
    end

    def test_argument_bool
      init_arguments
      assert_equal(true, @entry.args[5])
      @entry.set_arg_at(5, false)
      assert_equal(false, @entry.args[5])
      assert_exception(RuntimeError) {
	@entry.set_arg_at(5, nil)
      }
      assert_exception(RuntimeError) {
	@entry.set_arg_at(5, 'true')
      }
    end

    def test_argument_select
      init_arguments
      assert_equal('foo', @entry.args[6])
      @entry.set_arg_at(6, 'bar')
      assert_equal('bar', @entry.args[6])
      @entry.set_arg_at(6, 'foo') # allowed
      @entry.set_arg_at(6, 'bar') # allowed
      @entry.set_arg_at(6, 'baz') # allowed
      assert_exception(RuntimeError) {
	@entry.set_arg_at(6, 'quux') # not allowed
      }
      assert_exception(RuntimeError) {
	@entry.set_arg_at(6, nil) # not allowed
      }
    end

    def test_argument_radio
      init_arguments
      assert_equal('foo', @entry.args[7])
      @entry.set_arg_at(7, 'bar')
      assert_equal('bar', @entry.args[7])
      @entry.set_arg_at(7, 'foo') # allowed
      @entry.set_arg_at(7, 'bar') # allowed
      @entry.set_arg_at(7, 'baz') # allowed
      assert_exception(RuntimeError) {
	@entry.set_arg_at(7, 'quux') # not allowed
      }
      assert_exception(RuntimeError) {
	@entry.set_arg_at(7, nil) # not allowed
      }
    end

    def test_argument_checkset
      init_arguments
      assert_equal({ 'foo' => false,
		     'bar' => false,
		     'baz' => false
		   }, @entry.args[8])
      @entry.set_arg_at(8, {
			  'foo' => true,
			  'bar' => false,
			  'baz' => false
			})
      assert_equal({ 'foo' => true,
		     'bar' => false,
		     'baz' => false
		   }, @entry.args[8])
      assert_exception(RuntimeError) {
	@entry.set_arg_at(8, nil)
      }
      assert_exception(RuntimeError) {
	@entry.set_arg_at(8, 'checkset')
      }
      assert_exception(RuntimeError) {
	@entry.set_arg_at(8, { 'foo' => true })
      }
      assert_exception(RuntimeError) {
	@entry.set_arg_at(8, {
			    'foo' => true,
			    'bar' => false,
			    'baz' => false,
			    'qux' => false
			  })
      }
      assert_exception(RuntimeError) {
	@entry.set_arg_at(8, {
			    'foo' => false,
			    'bar' => false,
			    'baz' => nil
			  })
      }
    end
  end

  class TestDocumentList < RUNIT::TestCase
    include PseudoProperties

    def setup
      super

      # dependency configuration
      @rep = Rucy::Repository.new
      Rucy::Config.config(@rep)

      # for Rucy::DocumentFactory class
      @doc_names_call = 0
      @doc_args_call = 0

      @factory = Forwarder.new(self)
      class << @factory
	def_delegator :__getobj__, :doc_names
	def_delegator :__getobj__, :doc_args
      end

      # test target
      @doc_list = Rucy::DocumentList.new(@factory, @rep)
    end

    # for Rucy::DocumentFactory class

    def doc_names
      @doc_names_call += 1
      [ 'Page' ]
    end

    def doc_args(name)
      @doc_args_call += 1
      case (name)
      when 'Page'
	[ [ 'content', 'content', :text, nil ],
	  [ 'content_type', 'content-type', :string, 'text/html' ]
	]
      else
	raise "unknown document: #{name.inspect}"
      end
    end

    # test

    def test_modified_count
      @doc_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal(0, @doc_list.modified_count)
      @doc_list.modified_count = 1
      assert_equal(1, @doc_list.modified_count)
      assert_equal(0, @modified_count)
      @doc_list.save(@properties)
      assert_equal(1, @set_list_call)
      assert_equal(1, @modified_count)
    end

    def test_load_default_settings
      @doc_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal('documents', @list_name)
      assert(@doc_list.empty?)
      assert_equal(0, @doc_list.size)
      assert_equal(0, @doc_list.length)
      for mount_params in @doc_list
	assert_fail('found an element in empty list.')
      end
    end

    def test_load
      @store['documents'] = [
	{ 'document' => 'Page',
	  'arguments' => {
	    'content' => "Hello world.\n",
	    'content_type' => 'text/plain'
          },
	  'mount_path' => '/halo',
	  'mount_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => 'HALO'
	}
      ]
      @doc_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal('documents', @list_name)
      assert(! @doc_list.empty?)
      assert_equal(1, @doc_list.size)
      assert_equal(1, @doc_list.length)

      # test of operator `[]'
      assert_equal('Page', @doc_list[0].name)
      assert_equal([ "Hello world.\n",
		     'text/plain'
		   ], @doc_list[0].args)
      assert_equal([ { :name => 'content',
		       :type => :text,
		       :default => nil,
		       :value => "Hello world.\n"
		     },
		     { :name =>'content-type',
		       :type => :string,
		       :default => 'text/html',
		       :value => 'text/plain'
		     }
		   ], @doc_list[0].args2)
      assert_equal('/halo', @doc_list[0].path)
      assert_equal(nil, @doc_list[0].mask)
      assert_equal(nil, @doc_list[0].virtual_host)
      assert_equal('HALO', @doc_list[0].comment)

      # test of arguments modification
      @doc_list[0].args.clear
      assert_equal([ "Hello world.\n",
		     'text/plain'
		   ], @doc_list[0].args)

      # test of each
      count = 0
      for doc_params in @doc_list
	count += 1
	assert_equal('Page', doc_params.name)
	assert_equal([ "Hello world.\n",
		       'text/plain'
		     ], doc_params.args)
	assert_equal([ { :name => 'content',
			 :type => :text,
			 :default => nil,
			 :value => "Hello world.\n"
		       },
		       { :name =>'content-type',
			 :type => :string,
			 :default => 'text/html',
			 :value => 'text/plain'
		       }
		     ], doc_params.args2)
	assert_equal('/halo', doc_params.path)
	assert_equal(nil, doc_params.mask)
	assert_equal(nil, doc_params.virtual_host)
	assert_equal('HALO', doc_params.comment)

	# doc_args used at anywhere
	assert(@doc_args_call > 0)
      end
      assert_equal(1, count)
    end

    def test_load_backward_compatibility
      @store['documents'] = [
	{ 'document' => 'Page',
	  'arguments' => [
	    "Hello world.\n",
	    'text/plain'
          ],
	  'mount_path' => '/halo',
	  'mount_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => 'HALO'
	}
      ]
      @doc_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal('documents', @list_name)
      assert(! @doc_list.empty?)
      assert_equal(1, @doc_list.size)
      assert_equal(1, @doc_list.length)

      # test of operator `[]'
      assert_equal('Page', @doc_list[0].name)
      assert_equal([ "Hello world.\n",
		     'text/plain'
		   ], @doc_list[0].args)
      assert_equal([ { :name => 'content',
		       :type => :text,
		       :default => nil,
		       :value => "Hello world.\n"
		     },
		     { :name =>'content-type',
		       :type => :string,
		       :default => 'text/html',
		       :value => 'text/plain'
		     }
		   ], @doc_list[0].args2)
      assert_equal('/halo', @doc_list[0].path)
      assert_equal(nil, @doc_list[0].mask)
      assert_equal(nil, @doc_list[0].virtual_host)
      assert_equal('HALO', @doc_list[0].comment)

      # test of arguments modification
      @doc_list[0].args.clear
      assert_equal([ "Hello world.\n",
		     'text/plain'
		   ], @doc_list[0].args)

      # test of each
      count = 0
      for doc_params in @doc_list
	count += 1
	assert_equal('Page', doc_params.name)
	assert_equal([ "Hello world.\n",
		       'text/plain'
		     ], doc_params.args)
	assert_equal([ { :name => 'content',
			 :type => :text,
			 :default => nil,
			 :value => "Hello world.\n"
		       },
		       { :name =>'content-type',
			 :type => :string,
			 :default => 'text/html',
			 :value => 'text/plain'
		       }
		     ], doc_params.args2)
	assert_equal('/halo', doc_params.path)
	assert_equal(nil, doc_params.mask)
	assert_equal(nil, doc_params.virtual_host)
	assert_equal('HALO', doc_params.comment)

	# doc_args used at anywhere
	assert(@doc_args_call > 0)
      end
      assert_equal(1, count)
    end

    def test_save
      @store['documents'] = [
	{ 'document' => nil,
	  'arguments' => {},
	  'mount_path' => nil,
	  'mount_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	}
      ]
      @doc_list.load(@properties)
      @doc_list[0].name = 'Page'
      @doc_list[0].set_arg_at(0, "Hello world.\n")
      @doc_list[0].set_arg_at(1, 'text/plain')
      @doc_list[0].path = '/halo'
      @doc_list[0].mask = nil
      @doc_list[0].virtual_host = nil
      @doc_list[0].comment = 'HALO'
      @doc_list.save(@properties)
      assert_equal(1, @set_list_call)
      assert_equal('documents', @set_list_name)
      assert_equal([ { 'document' => 'Page',
		       'arguments' => {
			 'content' => "Hello world.\n",
			 'content_type' => 'text/plain'
		       },
		       'mount_path' => '/halo',
		       'mount_mask' => nil,
		       'virtual_host' => nil,
		       'comment' => 'HALO'
		     }
		   ], @store['documents'])
    end

    def test_save_with_nil_path
      @store['documents'] = [
	{ 'document' => nil,
	  'arguments' => {},
	  'mount_path' => nil,
	  'mount_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	}
      ]
      @doc_list.load(@properties)
      @doc_list[0].name = 'Page'
      @doc_list[0].set_arg_at(0, "Hello world.\n")
      @doc_list[0].set_arg_at(1, 'text/plain')
      @doc_list[0].path = nil	# invalid path!
      @doc_list[0].mask = nil
      @doc_list[0].virtual_host = nil
      @doc_list[0].comment = 'HALO'
      assert_exception(RuntimeError) { @doc_list.save(@properties) }
    end

    def test_save_with_invalid_path
      @store['documents'] = [
	{ 'document' => nil,
	  'arguments' => {},
	  'mount_path' => nil,
	  'mount_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	}
      ]
      @doc_list.load(@properties)
      @doc_list[0].name = 'Page'
      @doc_list[0].set_arg_at(0, "Hello world.\n")
      @doc_list[0].set_arg_at(1, 'text/plain')
      @doc_list[0].path = 'foo'	# invalid path!
      @doc_list[0].mask = nil
      @doc_list[0].virtual_host = nil
      @doc_list[0].comment = 'HALO'
      assert_exception(RuntimeError) { @doc_list.save(@properties) }
    end

    def test_add_entry
      @doc_list.load(@properties)
      assert(@doc_list.empty?)
      assert_equal(0, @doc_list.size)
      assert_equal(0, @doc_list.length)

      @doc_list.add_entry
      assert(! @doc_list.empty?)
      assert_equal(1, @doc_list.size)
      assert_equal(1, @doc_list.length)
      assert_equal(nil, @doc_list[0].name)
      #assert(@doc_list[0].args.empty?) # not access until being initialized
      assert_equal(nil, @doc_list[0].path)
      assert_equal(nil, @doc_list[0].mask)
      assert_equal(nil, @doc_list[0].virtual_host)
      assert_equal(nil, @doc_list[0].comment)

      @doc_list[0].init('Page')
      assert_equal(1, @doc_args_call)
      assert_equal('Page', @doc_list[0].name)
      assert_equal([ nil, 'text/html' ], @doc_list[0].args)
      assert_equal([ { :name => 'content',
		       :type => :text,
		       :default => nil,
		       :value => nil
		     },
		     { :name => 'content-type',
		       :type => :string,
		       :default => 'text/html',
		       :value => 'text/html'
		     }
		   ], @doc_list[0].args2)
      assert_equal(nil, @doc_list[0].path)
      assert_equal(nil, @doc_list[0].mask)
      assert_equal(nil, @doc_list[0].virtual_host)
      assert_equal(nil, @doc_list[0].comment)
    end

    def test_delete_at
      @store['documents'] = [
	{ 'document' => 'Foo',
	  'arguments' => {},
	  'mount_path' => nil,
	  'mount_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	}
      ]
      @doc_list.load(@properties)
      assert(! @doc_list.empty?)
      assert_equal(1, @doc_list.size)
      assert_equal(1, @doc_list.length)
      assert_equal('Foo', @doc_list[0].name)
      @doc_list.delete_at(0)
      assert(@doc_list.empty?)
      assert_equal(0, @doc_list.size)
      assert_equal(0, @doc_list.length)
    end

    def test_swap
      @store['documents'] = [
	{ 'document' => 'Foo',
	  'arguments' => {},
	  'mount_path' => nil,
	  'mount_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	},
	{ 'document' => 'Bar',
	  'arguments' => {},
	  'mount_path' => nil,
	  'mount_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	}
      ]
      @doc_list.load(@properties)
      assert_equal('Foo', @doc_list[0].name)
      assert_equal('Bar', @doc_list[1].name)
      @doc_list.swap(0, 1)
      assert_equal('Bar', @doc_list[0].name)
      assert_equal('Foo', @doc_list[1].name)
    end
  end

  class TestFilterList < RUNIT::TestCase
    include PseudoProperties

    def setup
      super

      # dependency configuration
      @rep = Rucy::Repository.new
      Rucy::Config.config(@rep)

      # for Rucy::DocumentFactory class
      @filter_names_call = 0
      @filter_args_call = 0

      @factory = Forwarder.new(self)
      class << @factory
	def_delegator :__getobj__, :filter_names
	def_delegator :__getobj__, :filter_args
      end

      # test target
      @filter_list = Rucy::FilterList.new(@factory, @rep)
    end

    # for Rucy::DocumentFactory class

    def filter_names
      @filter_names_call += 1
      'Foo'
    end

    def filter_args(name)
      @filter_args_call += 1
      case (name)
      when 'Foo'
	[ [ 'first_arg', 'first argument', :string, 'alice' ],
	  [ 'second_arg', 'second argument', :regexp, /bob/ ]
	]
      else
	raise "unknown filter: #{name.inspect}"
      end
    end

    # test

    def test_modified_count
      @filter_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal(0, @filter_list.modified_count)
      @filter_list.modified_count = 1
      assert_equal(1, @filter_list.modified_count)
      assert_equal(0, @modified_count)
      @filter_list.save(@properties)
      assert_equal(1, @set_list_call)
      assert_equal(1, @modified_count)
    end

    def test_load_default_settings
      @filter_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal('filters', @list_name)
      assert(@filter_list.empty?)
      assert_equal(0, @filter_list.size)
      assert_equal(0, @filter_list.length)
      for mount_params in @filter_list
	assert_fail('found an element in empty list.')
      end
    end

    def test_load
      @store['filters'] = [
	{ 'filter' => 'Foo',
	  'arguments' => {
	    'first_arg' => 'apple',
	    'second_arg' => /banana/
          },
	  'attach_path' => '/foo',
	  'attach_mask' => %r"\.html?($|/)",
	  'virtual_host' => 'www.foo.org',
	  'comment' => 'test'
	}
      ]
      @filter_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal('filters', @list_name)
      assert(! @filter_list.empty?)
      assert_equal(1, @filter_list.size)
      assert_equal(1, @filter_list.length)

      # test of operator `[]'
      assert_equal('Foo', @filter_list[0].name)
      assert_equal([ 'apple', /banana/ ], @filter_list[0].args)
      assert_equal([ { :name => 'first argument',
		       :type => :string,
		       :default => 'alice',
		       :value => 'apple'
		     },
		     { :name => 'second argument',
		       :type => :regexp,
		       :default => /bob/,
		       :value => /banana/
		     }
		   ], @filter_list[0].args2)
      assert_equal('/foo', @filter_list[0].path)
      assert_equal(%r"\.html?($|/)", @filter_list[0].mask)
      assert_equal('www.foo.org', @filter_list[0].virtual_host)
      assert_equal('test', @filter_list[0].comment)

      # test of arguments modification
      @filter_list[0].args.clear
      assert_equal([ 'apple', /banana/ ], @filter_list[0].args)

      # test of each
      count = 0
      for filter_params in @filter_list
	count += 1
	assert_equal('Foo', filter_params.name)
	assert_equal([ 'apple', /banana/ ], filter_params.args)
	assert_equal([ { :name => 'first argument',
			 :type => :string,
			 :default => 'alice',
			 :value => 'apple'
		       },
		       { :name => 'second argument',
			 :type => :regexp,
			 :default => /bob/,
			 :value => /banana/
		       }
		     ], filter_params.args2)
	assert_equal('/foo', filter_params.path)
	assert_equal(%r"\.html?($|/)", filter_params.mask)
	assert_equal('www.foo.org', filter_params.virtual_host)
	assert_equal('test', filter_params.comment)

	# filter_args used at anywhere
	assert(@filter_args_call > 0)
      end
      assert_equal(1, count)
    end

    def test_load_backward_compatibility
      @store['filters'] = [
	{ 'filter' => 'Foo',
	  'arguments' => [
	    'apple',
	    /banana/
          ],
	  'attach_path' => '/foo',
	  'attach_mask' => %r"\.html?($|/)",
	  'virtual_host' => 'www.foo.org',
	  'comment' => 'test'
	}
      ]
      @filter_list.load(@properties)
      assert_equal(1, @list_call)
      assert_equal('filters', @list_name)
      assert(! @filter_list.empty?)
      assert_equal(1, @filter_list.size)
      assert_equal(1, @filter_list.length)

      # test of operator `[]'
      assert_equal('Foo', @filter_list[0].name)
      assert_equal([ 'apple', /banana/ ], @filter_list[0].args)
      assert_equal([ { :name => 'first argument',
		       :type => :string,
		       :default => 'alice',
		       :value => 'apple'
		     },
		     { :name => 'second argument',
		       :type => :regexp,
		       :default => /bob/,
		       :value => /banana/
		     }
		   ], @filter_list[0].args2)
      assert_equal('/foo', @filter_list[0].path)
      assert_equal(%r"\.html?($|/)", @filter_list[0].mask)
      assert_equal('www.foo.org', @filter_list[0].virtual_host)
      assert_equal('test', @filter_list[0].comment)

      # test of arguments modification
      @filter_list[0].args.clear
      assert_equal([ 'apple', /banana/ ], @filter_list[0].args)

      # test of each
      count = 0
      for filter_params in @filter_list
	count += 1
	assert_equal('Foo', filter_params.name)
	assert_equal([ 'apple', /banana/ ], filter_params.args)
	assert_equal([ { :name => 'first argument',
			 :type => :string,
			 :default => 'alice',
			 :value => 'apple'
		       },
		       { :name => 'second argument',
			 :type => :regexp,
			 :default => /bob/,
			 :value => /banana/
		       }
		     ], filter_params.args2)
	assert_equal('/foo', filter_params.path)
	assert_equal(%r"\.html?($|/)", filter_params.mask)
	assert_equal('www.foo.org', filter_params.virtual_host)
	assert_equal('test', filter_params.comment)

	# filter_args used at anywhere
	assert(@filter_args_call > 0)
      end
      assert_equal(1, count)
    end

    def test_save
      @store['filters'] = [
	{ 'filter' => nil,
	  'arguments' => {},
	  'attach_path' => nil,
	  'attach_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	}
      ]
      @filter_list.load(@properties)
      @filter_list[0].name = 'Foo'
      @filter_list[0].set_arg_at(0, 'apple')
      @filter_list[0].set_arg_at(1, /banana/)
      @filter_list[0].path = '/foo'
      @filter_list[0].mask = %r"\.html($|/)"
      @filter_list[0].virtual_host = 'www.foo.org'
      @filter_list[0].comment = 'test'
      @filter_list.save(@properties)
      assert_equal(1, @set_list_call)
      assert_equal('filters', @set_list_name)
      assert_equal([ { 'filter' => 'Foo',
		       'arguments' => {
			 'first_arg' => 'apple',
			 'second_arg' => /banana/
		       },
		       'attach_path' => '/foo',
		       'attach_mask' => %r"\.html($|/)",
		       'virtual_host' => 'www.foo.org',
		       'comment' => 'test'
		     }
		   ], @store['filters'])
    end

    def test_save_with_nil_path
      @store['filters'] = [
	{ 'filter' => nil,
	  'arguments' => {},
	  'attach_path' => nil,
	  'attach_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	}
      ]
      @filter_list.load(@properties)
      @filter_list[0].name = 'Foo'
      @filter_list[0].set_arg_at(0, 'apple')
      @filter_list[0].set_arg_at(1, /banana/)
      @filter_list[0].path = nil # invalid path!
      @filter_list[0].mask = %r"\.html($|/)"
      @filter_list[0].virtual_host = 'www.foo.org'
      @filter_list[0].comment = 'test'
      assert_exception(RuntimeError) { @filter_list.save(@properties) }
    end

    def test_save_with_invalid_path
      @store['filters'] = [
	{ 'filter' => nil,
	  'arguments' => {},
	  'attach_path' => nil,
	  'attach_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	}
      ]
      @filter_list.load(@properties)
      @filter_list[0].name = 'Foo'
      @filter_list[0].set_arg_at(0, 'apple')
      @filter_list[0].set_arg_at(1, /banana/)
      @filter_list[0].path = 'foo' # invalid path!
      @filter_list[0].mask = %r"\.html($|/)"
      @filter_list[0].virtual_host = 'www.foo.org'
      @filter_list[0].comment = 'test'
      assert_exception(RuntimeError) { @filter_list.save(@properties) }
    end

    def test_add_entry
      @filter_list.load(@properties)
      assert(@filter_list.empty?)
      assert_equal(0, @filter_list.size)
      assert_equal(0, @filter_list.length)

      @filter_list.add_entry
      assert(! @filter_list.empty?)
      assert_equal(1, @filter_list.size)
      assert_equal(1, @filter_list.length)
      assert_equal(nil, @filter_list[0].name)
      #assert(@filter_list[0].args.empty?) # not access until being initialized
      assert_equal(nil, @filter_list[0].path)
      assert_equal(nil, @filter_list[0].mask)
      assert_equal(nil, @filter_list[0].virtual_host)
      assert_equal(nil, @filter_list[0].comment)

      @filter_list[0].init('Foo')
      assert_equal(1, @filter_args_call)
      assert_equal('Foo', @filter_list[0].name)
      assert_equal([ 'alice', /bob/ ], @filter_list[0].args)
      assert_equal([ { :name => 'first argument',
		       :type => :string,
		       :default => 'alice',
		       :value => 'alice'
		     },
		     { :name => 'second argument',
		       :type => :regexp,
		       :default => /bob/,
		       :value => /bob/
		     }
		   ], @filter_list[0].args2)
      assert_equal(nil, @filter_list[0].path)
      assert_equal(nil, @filter_list[0].mask)
      assert_equal(nil, @filter_list[0].virtual_host)
      assert_equal(nil, @filter_list[0].comment)
    end

    def test_delete_at
      @store['filters'] = [
	{ 'filter' => 'Foo',
	  'arguments' => {},
	  'attach_path' => nil,
	  'attach_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	}
      ]
      @filter_list.load(@properties)
      assert(! @filter_list.empty?)
      assert_equal(1, @filter_list.size)
      assert_equal(1, @filter_list.length)
      assert_equal('Foo', @filter_list[0].name)
      @filter_list.delete_at(0)
      assert(@filter_list.empty?)
      assert_equal(0, @filter_list.size)
      assert_equal(0, @filter_list.length)
    end

    def test_swap
      @store['filters'] = [
	{ 'filter' => 'Foo',
	  'arguments' => {},
	  'attach_path' => nil,
	  'attach_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	},
	{ 'filter' => 'Bar',
	  'arguments' => {},
	  'attach_path' => nil,
	  'attach_mask' => nil,
	  'virtual_host' => nil,
	  'comment' => nil
	}
      ]
      @filter_list.load(@properties)
      assert_equal('Foo', @filter_list[0].name)
      assert_equal('Bar', @filter_list[1].name)
      @filter_list.swap(0, 1)
      assert_equal('Bar', @filter_list[0].name)
      assert_equal('Foo', @filter_list[1].name)
    end
  end
end
