#!/usr/local/bi/ruby
# $Id: test_rc_d.rb,v 1.6 2005/02/11 16:25:53 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'rucy/rc_d'

module TestRucy
  class TestProcessProbe < RUNIT::TestCase
    def setup
      # target
      @probe = Rucy::ProcessProbe # module
    end

    def test_proc_alive?
      assert((@probe.proc_alive? $$))
      alive_proc_count = 0
      dead_proc_count = 0
      10.times do
	system('ruby -e0')
      end
      for pid in (($$ - 100)..($$ + 100))
	if (@probe.proc_alive? pid) then
	  alive_proc_count += 1
	else
	  dead_proc_count += 1
	end
      end
      assert(alive_proc_count > 0)
      assert(dead_proc_count > 0)
      assert(100, alive_proc_count + dead_proc_count)
    end
  end

  class TestRC_d < RUNIT::TestCase
    SELF_PID = $$
    ALIVE_PID = $$ - 1
    DEAD_PID = $$ + 1

    def setup
      # test repository
      @rep = Rucy::Repository.new

      # for Rucy::Daemon module
      @daemon_call = 0

      @Daemon = Forwarder.new(self)
      class << @Daemon
	def_delegator :__getobj__, :daemon
      end
      @rep.set_value(:Daemon_module, @Daemon)

      # for Rucy::ProcessProbe module
      @proc_alive_call = 0
      @proc_alive_pid = nil

      @ProcessProbe = Forwarder.new(self)
      class << @ProcessProbe
	def_delegator :__getobj__, :proc_alive?
      end
      @rep.set_value(:ProcessProbe_module, @ProcessProbe)

      # for File class
      @open_call = 0
      @open_path = nil
      @open_mode = nil
      @open_block_given_p = nil
      @delete_call = 0
      @delete_path = nil

      @File = Forwarder.new(self)
      class << @File
	def_delegator :__getobj__, :open
	def_delegator :__getobj__, :delete
      end
      @rep.set_value(:File_class, @File)

      # for Signal module
      @trap_call = 0
      @trap_sig = Hash.new

      @Signal = Forwarder.new(self)
      class << @Signal
	def_delegator :__getobj__, :trap
      end
      @rep.set_value(:Signal_module, @Signal)

      # for Process module
      @kill_call = 0
      @kill_sig = nil
      @kill_pid = nil

      @Process = Forwarder.new(self)
      class << @Process
	def_delegator :__getobj__, :kill
      end
      @rep.set_value(:Process_module, @Process)

      # for Rucy::Server class
      @server_daemon_call = 0
      @server_daemon_flag = nil
      @restart_call = 0
      @close_call = 0

      @server = Forwarder.new(self)
      class << @server
	def_delegator :__getobj__, :server_daemon, :daemon
	def_delegator :__getobj__, :restart
	def_delegator :__getobj__, :close
      end

      # target
      @store_path = 'rc_d.pid'
      @rc_d = Rucy::RC_d.new(@store_path, @rep)
    end

    def teardown
      File.delete(@store_path) if (File.exist? @store_path)
    end

    # for Rucy::Daemon module

    def daemon
      @daemon_call += 1
      nil
    end

    # for Rucy::ProcessProbe module

    def proc_alive?(pid)
      @proc_alive_call += 1
      @proc_alive_pid = pid
      case (pid)
      when SELF_PID, ALIVE_PID
	true
      else
	false
      end
    end

    # for Signal module

    def trap(sig, &block)
      @trap_call += 1
      @trap_sig[sig] = block
      nil
    end

    # for File class

    def open(path, mode, &block)
      @open_call += 1
      @open_path = path
      @open_mode = mode
      @open_block_given_p = block_given?
      File.open(path, mode, &block)
    end

    def delete(path)
      @delete_call += 1
      @delete_path = path
      File.delete(path)
    end

    # for Process module

    def kill(sig, pid)
      @kill_call += 1
      @kill_sig = sig
      @kill_pid = pid
      nil
    end

    # for Rucy::Server class

    def server_daemon
      @server_daemon_call += 1
      @server_daemon_flag
    end

    def restart
      @restart_call += 1
      nil
    end

    def close
      @close_call += 1
      nil
    end

    # test

    def test_write_pid
      @rc_d.write_pid
      assert_equal(1, @open_call)
      assert_equal(@store_path, @open_path)
      assert_equal(File::WRONLY | File::CREAT | File::TRUNC, @open_mode)
      assert_equal(true, @open_block_given_p)
      assert((File.exist? @store_path))
      assert_equal(SELF_PID.to_s + "\n", File.open(@store_path) {|input| input.read })
      assert_equal(0, @proc_alive_call)
    end

    def test_write_pid_overwrite_alive_process
      File.open(@store_path, 'w') {|output| output.write(ALIVE_PID.to_s + "\n") }
      @rc_d.write_pid
      assert_equal(1, @open_call)
      assert_equal(@store_path, @open_path)
      assert_equal(File::WRONLY | File::CREAT | File::TRUNC, @open_mode)
      assert_equal(true, @open_block_given_p)
      assert((File.exist? @store_path))
      assert_equal(SELF_PID.to_s + "\n", File.open(@store_path) {|input| input.read })
      assert_equal(0, @proc_alive_call)
    end

    def test_write_pid_overwrite_dead_process
      File.open(@store_path, 'w') {|output| output.write(DEAD_PID.to_s + "\n") }
      @rc_d.write_pid
      assert_equal(1, @open_call)
      assert_equal(@store_path, @open_path)
      assert_equal(File::WRONLY | File::CREAT | File::TRUNC, @open_mode)
      assert_equal(true, @open_block_given_p)
      assert((File.exist? @store_path))
      assert_equal(SELF_PID.to_s + "\n", File.open(@store_path) {|input| input.read })
      assert_equal(0, @proc_alive_call)
    end

    def test_lock
      @rc_d.lock
      assert_equal(1, @open_call)
      assert_equal(@store_path, @open_path)
      assert_equal(File::WRONLY | File::CREAT | File::TRUNC | File::EXCL, @open_mode)
      assert_equal(true, @open_block_given_p)
      assert((File.exist? @store_path))
      assert_equal(SELF_PID.to_s + "\n", File.open(@store_path) {|input| input.read })
      assert_equal(0, @proc_alive_call)
    end

    def test_locked_by_alive_process
      File.open(@store_path, 'w') {|output| output.write(ALIVE_PID.to_s + "\n") }
      assert_exception(RuntimeError) { @rc_d.lock }
      assert_equal(2, @open_call)
      assert_equal(@store_path, @open_path)
      assert_equal('r', @open_mode)
      assert_equal(true, @open_block_given_p)
      assert_equal(ALIVE_PID.to_s + "\n", File.open(@store_path) {|input| input.read })
      assert_equal(1, @proc_alive_call)
      assert_equal(ALIVE_PID, @proc_alive_pid)
    end

    def test_locked_by_dead_process
      File.open(@store_path, 'w') {|output| output.write(DEAD_PID.to_s + "\n") }
      @rc_d.lock
      assert_equal(1, @delete_call)
      assert_equal(@store_path, @delete_path)
      assert_equal(3, @open_call)
      assert_equal(@store_path, @open_path)
      assert_equal(File::WRONLY | File::CREAT | File::TRUNC | File::EXCL, @open_mode)
      assert_equal(true, @open_block_given_p)
      assert_equal(SELF_PID.to_s + "\n", File.open(@store_path) {|input| input.read })
      assert_equal(1, @proc_alive_call)
      assert_equal(DEAD_PID, @proc_alive_pid)
    end

    def test_running
      assert(! (File.exist? @store_path))
      assert_equal(false, @rc_d.running?)
      File.open(@store_path, 'w') {|output| output.write(ALIVE_PID.to_s + "\n") }
      assert_equal(true, @rc_d.running?)
      assert_equal(1, @proc_alive_call)
      assert_equal(ALIVE_PID, @proc_alive_pid)
      File.open(@store_path, 'w') {|output| output.write(DEAD_PID.to_s + "\n") }
      assert_equal(false, @rc_d.running?)
      assert_equal(2, @proc_alive_call)
      assert_equal(DEAD_PID, @proc_alive_pid)
    end

    def test_stop
      File.open(@store_path, 'w') {|output| output.write(ALIVE_PID.to_s + "\n") }
      @rc_d.stop
      assert_equal(1, @kill_call)
      assert_equal('TERM', @kill_sig)
      assert_equal(ALIVE_PID, @kill_pid)
    end

    def test_server_setup
      @server_daemon_flag = false
      @rc_d.server_setup(@server)
      assert_equal(1, @server_daemon_call)
      assert_equal(1, @trap_call)
      @trap_sig['TERM'].call
      assert_equal(1, @close_call)
      assert_equal(0 , @daemon_call)
    end

    def test_server_setup_as_daemon
      @server_daemon_flag = true
      @rc_d.server_setup(@server)
      assert_equal(1, @server_daemon_call)
      assert_equal(2, @trap_call)
      @trap_sig['HUP'].call
      assert_equal(1, @restart_call)
      @trap_sig['TERM'].call
      assert_equal(1, @close_call)
      assert_equal(1 , @daemon_call)
    end
  end
end
