#!/usr/local/bin/ruby
# $Id: test_rep.rb,v 1.1 2004/12/13 15:59:39 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'rucy/rep'

module TestRucy
  class TestRepository < RUNIT::TestCase
    def setup
      # value object
      @value = Object.new

      # class object
      @new_call = 0
      @new_arg1 = nil
      @new_arg2 = nil
      @new_iterator_p = nil

      # for block
      @hook_call = 0
      @hook_arg1 = nil
      @hook_arg2 = nil

      @class = Forwarder.new(self)
      class << @class
        def_delegator :__getobj__, :new
      end

      # target
      @rep = Rucy::Repository.new
    end

    # for class object

    def new(arg1, arg2)
      @new_call += 1
      @new_arg1 = arg1
      @new_arg2 = arg2
      @new_iterator_p = iterator?
      @value
    end

    # for block

    def hook(arg1, arg2)
      @hook_call += 1
      @hook_arg1 = arg1
      @hook_arg2 = arg2
      nil
    end

    # test

    def test_not_found
      assert_exception(RuntimeError) {
	@rep.fetch(:Nothing)
      }
    end

    def test_value
      @rep.set_value(:Value, @value)
      assert_equal(@value, @rep.fetch(:Value))
    end

    def test_class_with_block
      @rep.set_class(:Class, @class)
      assert_equal(@value, @rep.fetch(:Class, 'foo', :bar) {})
      assert_equal(1, @new_call)
      assert_equal('foo', @new_arg1)
      assert_equal(:bar, @new_arg2)
      assert_equal(true, @new_iterator_p)
    end

    def test_class_without_block
      @rep.set_class(:Class, @class)
      assert_equal(@value, @rep.fetch(:Class, 'foo', :bar))
      assert_equal(1, @new_call)
      assert_equal('foo', @new_arg1)
      assert_equal(:bar, @new_arg2)
      assert_equal(false, @new_iterator_p)
    end

    def test_block
      @rep.set_block(:Block) {|arg1, arg2|
	hook(arg1, arg2)
	@value
      }
      assert_equal(@value, @rep.fetch(:Block, 'foo', :bar))
      assert_equal(1, @hook_call)
      assert_equal('foo', @hook_arg1)
      assert_equal(:bar, @hook_arg2)
    end
  end
end
