#!/usr/local/bin/ruby
# $Id: test_server.rb,v 1.14 2005/05/31 15:43:49 toki Exp $

require 'rubyunit'
require 'forwarder'
require 'thread'
require 'socket'
require 'sockutils'
require 'rucy/rep'
#require 'rucy/config'
require 'rucy/logger'
require 'rucy/messenger'
require 'rucy/server'
require 'rucy/document'
require 'rucy/response'

Thread.abort_on_exception = true

module TestRucy
  class TestMultiThreadServer < RUNIT::TestCase
    include SocketUtils

    DEBUG_LOGGING = false

    def setup
      # dependency configuration
      @rep = Rucy::Repository.new
      @rep.set_value(:MultiThreadMessenger_class, Rucy::MultiThreadMessenger)
      @rep.set_class(:TCPServer, TCPServer)

      # for Rucy::Document class
      @document_open_call = 0
      @document_close_call = 0

      @document = Forwarder.new(self)
      class << @document
	def_delegator :__getobj__, :open
	def_delegator :__getobj__, :close
	def_delegator :__getobj__, :each
      end

      @restart_called = 0
      @server_open_call = 0
      @server_close_call = 0
      @server = Rucy::Server.new(@rep)
      @server.open_hook{|s|
	@server_open_call += 1
	s.bind_address = 'localhost'
	s.port = 0
	s.mount(Rucy::Page.new("<html>Hello world.</html>\n"), '/')
	s.mount(@document, '/test_document_open_close')
	if (DEBUG_LOGGING) then
	  logger = Rucy::Logger.new(STDERR)
	  logger.level = Rucy::LoggingLevel::LOG_DEBUG
	  s.add_logger(logger)
	  s.logging_access
	end
      }
      @server.close_hook{|s|
	@server_close_call += 1
      }
      @thread = Thread.new{
	@server.accept
      }
      begin
	sleep(0.1)
      end while (@server.port == 0)
      @port = @server.port
    end

    def teardown
      @server.close
      @thread.join
      assert_equal(1 + @restart_called, @server_open_call)
      assert_equal(1 + @restart_called, @server_close_call)
      assert_equal(1 + @restart_called, @document_open_call)
      assert_equal(1 + @restart_called, @document_close_call)
    end

    # for Rucy::Document class

    def open
      @document_open_call += 1
      nil
    end

    def close
      @document_close_call += 1
      nil
    end

    def each(traversed={})
      yield(@document)
      nil
    end

    # test

    def test_restart
      @server.restart
      @restart_called += 1
      sleep(1)
    end

    def test_GET
      socket = TCPSocket.new('localhost', @port)
      setsockopt(socket)
      begin
	socket.print "GET / HTTP/1.1\r\n"
	socket.print "Host: localhost:#{@port}\r\n"
	socket.print "\r\n"
	socket.flush

	response = Rucy::Response.new
	response.parse(socket)
	assert_equal('HTTP/1.1 200 OK', response.line)
	assert(! response.headers('Connection').find{|v| v =~ /close/i })
	assert_equal('26', response.header('Content-Length'))
	assert_equal('text/html', response.header('Content-Type'))
	assert_match(response.header('Date'), /^(Mon|Tue|Wed|Thu|Fri|Sat|Sun), \d{2} (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) \d{4} \d{2}:\d{2}:\d{2} GMT$/)
	assert_equal(Rucy::SERVER_TOKEN_LIST, response.header('Server'))
	assert_equal("<html>Hello world.</html>\n", socket.read(26))

	socket.print "GET / HTTP/1.1\r\n"
	socket.print "Host: localhost:#{@port}\r\n"
	socket.print "Connection: close\r\n"
	socket.print "\r\n"
	socket.flush

	response = Rucy::Response.new
	response.parse(socket)
	assert_equal('HTTP/1.1 200 OK', response.line)
	assert_equal('close', response.header('Connection'))
	assert_equal('26', response.header('Content-Length'))
	assert_equal('text/html', response.header('Content-Type'))
	assert_match(response.header('Date'), /^(Mon|Tue|Wed|Thu|Fri|Sat|Sun), \d{2} (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) \d{4} \d{2}:\d{2}:\d{2} GMT$/)
	assert_equal(Rucy::SERVER_TOKEN_LIST, response.header('Server'))
	assert_equal("<html>Hello world.</html>\n", socket.read(26))
      ensure
	socket.close
      end
    end

    def make_access_thread(nth, nreqs, conn_wait, conn_next, start_wait)
      Thread.new{
	conn_wait.call(nth)
	socket = TCPSocket.new('localhost', @port)
	setsockopt(socket)
	conn_next.call
	start_wait.call
	begin
	  (nreqs - 1).times do
	    socket.print "GET / HTTP/1.1\r\n"
	    socket.print "Host: localhost:#{@port}\r\n"
	    socket.print "\r\n"

	    response = Rucy::Response.new
	    response.parse(socket)
	    assert_equal('HTTP/1.1 200 OK', response.line)
	    assert(! response.headers('Connection').find{|v| v =~ /close/i })
	    assert_equal('26', response.header('Content-Length'))
	    assert_equal('text/html', response.header('Content-Type'))
	    assert_match(response.header('Date'), /^(Mon|Tue|Wed|Thu|Fri|Sat|Sun), \d{2} (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) \d{4} \d{2}:\d{2}:\d{2} GMT$/)
	    assert_equal(Rucy::SERVER_TOKEN_LIST, response.header('Server'))
	    assert_equal("<html>Hello world.</html>\n", socket.read(26))
	  end

	  socket.print "GET / HTTP/1.1\r\n"
	  socket.print "Host: localhost:#{@port}\r\n"
	  socket.print "Connection: close\r\n"
	  socket.print "\r\n"

	  response = Rucy::Response.new
	  response.parse(socket)
	  assert_equal('HTTP/1.1 200 OK', response.line)
	  assert_equal('close', response.header('Connection'))
	  assert_equal('26', response.header('Content-Length'))
	  assert_equal('text/html', response.header('Content-Type'))
	  assert_match(response.header('Date'), /^(Mon|Tue|Wed|Thu|Fri|Sat|Sun), \d{2} (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) \d{4} \d{2}:\d{2}:\d{2} GMT$/)
	  assert_equal(Rucy::SERVER_TOKEN_LIST, response.header('Server'))
	  assert_equal("<html>Hello world.</html>\n", socket.read(26))
	ensure
	  socket.close
	end
      }
    end
    private :make_access_thread

    def test_multi_access
      conn_count = 0
      conn_lock = Mutex.new
      conn_cond = ConditionVariable.new
      conn_wait = proc{|nth|
	conn_lock.synchronize{
	  until (conn_count == nth)
	    conn_cond.wait(conn_lock)
	  end
	}
      }
      conn_next = proc{
	sleep(0.1)		# wait time for not causing overflow of backlog
	conn_lock.synchronize{
	  conn_count += 1
	  conn_cond.broadcast
	}
      }

      start_flag = false
      start_lock = Mutex.new
      start_cond = ConditionVariable.new
      start_wait = proc{
	start_lock.synchronize{
	  until (start_flag)
	    start_cond.wait(start_lock)
	  end
	}
      }
      start_go = proc{
	start_lock.synchronize{
	  start_flag = true
	  start_cond.broadcast
	}
      }

      th_grp = ThreadGroup.new
      nconns = 16		# less than total queue length
      nreqs = 32
      nconns.times do |i|
	th_grp.add(make_access_thread(i, nreqs, conn_wait, conn_next, start_wait))
      end
      conn_wait.call(nconns)
      start_go.call

      for th in th_grp.list
	th.join
      end
    end
  end

  class TestMultiThreadServerTimeout < RUNIT::TestCase
    include SocketUtils

    DEBUG_LOGGING = false

    def setup
      # dependency configuration
      @rep = Rucy::Repository.new
      @rep.set_value(:MultiThreadMessenger_class, Rucy::MultiThreadMessenger)
      @rep.set_class(:TCPServer, TCPServer)

      # for Rucy::Document class
      @document_open_call = 0
      @document_close_call = 0

      @document = Forwarder.new(self)
      class << @document
	def_delegator :__getobj__, :open
	def_delegator :__getobj__, :close
	def_delegator :__getobj__, :each
      end

      @server_open_call = 0
      @server_close_call = 0
      @server = Rucy::Server.new(@rep)
      @server.open_hook{|s|
	@server_open_call += 1
	s.bind_address = 'localhost'
	s.port = 0
	s.timeout = 0.01	# short timeout for test
	s.mount(Rucy::Page.new("<html>Hello world.</html>\n"), '/')
	s.mount(@document, '/test_document_open_close')
	if (DEBUG_LOGGING) then
	  logger = Rucy::Logger.new(STDERR)
	  logger.level = Rucy::LoggingLevel::LOG_DEBUG
	  s.add_logger(logger)
	  s.logging_access
	end
      }
      @server.close_hook{|s|
	@server_close_call += 1
      }
      @thread = Thread.new{
	@server.accept
      }
      begin
	sleep(0.1)
      end while (@server.port == 0)
      @port = @server.port
    end

    def teardown
      @server.close
      @thread.join
      assert_equal(1, @server_open_call)
      assert_equal(1, @server_close_call)
      assert_equal(1, @document_open_call)
      assert_equal(1, @document_close_call)
    end

    # for Rucy::Document class

    def open
      @document_open_call += 1
      nil
    end

    def close
      @document_close_call += 1
      nil
    end

    def each(traversed={})
      yield(@document)
      nil
    end

    # test

    def test_timeout
      socket = TCPSocket.new('localhost', @port)
      setsockopt(socket)
      begin
	sleep(0.1)		# wait for timeout
	assert_nil(socket.gets)
      ensure
	socket.close
      end
    end
  end
end
